from pydantic import BaseModel, EmailStr
from typing import List, Optional

# RCIC
class Rcic(BaseModel):
    first_name: str
    last_name: str
    company: str
    sex: str
    rcic_number: str

    @property
    def name(self):
        return self.first_name + " " + self.last_name


class RcicList(BaseModel):
    id_name: str
    first_name: str
    last_name: str
    employer_legal_name: str
    country_code: str
    phone: str
    email: EmailStr
    unit: Optional[str]
    street_number: str
    street_name: str
    city: str
    province: str
    country: str
    post_code: str
    rcic_number: str

    @property
    def name(self):
        return self.first_name + " " + self.last_name

    @property
    def line1(self):
        l1 = self.unit + " " if self.unit else ""
        l1 += self.street_number + " " if self.street_number else ""
        l1 += self.street_name
        return l1


class Rcics:
    def __init__(self, rcics: List[RcicList]) -> None:
        self.rcics = rcics

    def getRcicByName(self, first_name: str, last_name: str):
        rcics = [
            rcic
            for rcic in self.rcics
            if rcic.first_name.lower() == first_name.lower()
            and rcic.last_name.lower() == last_name.lower()
        ]
        return rcics[0] if len(rcics) > 0 else None

    def getRcicByNumber(self, rcic_number: str):
        rcics = [
            rcic
            for rcic in self.rcics
            if rcic.rcic_number.lower() == rcic_number.lower()
        ]
        return rcics[0] if len(rcics) > 0 else None

    def getRcicByIdName(self, id_name: str):
        rcics = [rcic for rcic in self.rcics if rcic.id_name.lower() == id_name.lower()]
        return rcics[0] if len(rcics) > 0 else None



from datetime import date
from pydantic import BaseModel, EmailStr
from typing import List, Optional
from datetime import date
from model.common.commonmodel import CommonModel, BuilderModel
from model.common.rcic import RcicList
from model.common.contact import ContactBase
from pdfform.bcpnp.fbrep import FormBuilderBcRep
import json
from termcolor import colored
from model.common.employerbase import EmployerBase
from webform.bcpnp.rep import Representative


class General(EmployerBase):
    pass


class Contact(ContactBase):
    pass


class Personal(BaseModel):
    last_name: str
    first_name: str
    sex: str
    dob: date
    uci: Optional[str]


class MRepModel(BaseModel, BuilderModel):
    rciclist: List[RcicList]
    personal: Personal
    general: General
    contact: List[Contact]

    def make_pdf_form(self, output_json, rcic_id_name, *args, **kwargs):
        pf = FormBuilderBcRep(self, rcic_id_name)
        form = pf.get_form()
        with open(output_json, "w") as output:
            json.dump(form.actions, output, indent=3, default=str)
        print(colored(f"{output_json} has been created. ", "green"))

    def make_web_form(self, output_json, upload_dir, rcic_id_name, *args, **kwargs):
        # actions = Representative(self, rcic_id_name).fill()
        # with open(output_json, "w") as output:
        #     json.dump(actions, output, indent=3, default=str)
        # print(colored(f"{output_json} has been created. ", "green"))
        raise ValueError("This model doesn't have webform...")

    def context(self, *args, **kwargs):
        raise ValueError("This model doesn't have webform...")


class MRepModelE(CommonModel, MRepModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/pa.xlsx", "excel/rep.xlsx", "excel/er.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from pdfform.tr.common import TrCommon
from pdfform.tr import option_lists


class FormBuilder5710(TrCommon):
    """Form builder"""

    def start(self):
        """Add initial 3 skips"""
        self.form.add_skip(3)

    def add_header(self):
        """Add header section items"""
        personal = self.applicant.personal
        self.form.add_text(personal.uci)
        self.form.add_dropdown(
            self.applicant.trcasein.service_in, option_lists.service_language
        )

        # apply WP for same employer, apply WP for new employer,restore status as worker,, TRP with new employer
        if self.applicant.wpincanada.application_purpose == "restore status as worker":
            self.form.add_checkbox(True)  # pick apply for wp with same employer
            self.form.add_skip(1)
            self.form.add_checkbox(True)  # pick restore status
            self.form.add_skip(1)
        elif self.applicant.wpincanada.application_purpose == "TRP with same employer":
            self.form.add_checkbox(True)  # pick apply for wp with same employer
            self.form.add_skip(2)
            self.form.add_checkbox(True)  # pick restore status
        elif self.applicant.wpincanada.application_purpose == "TRP with new employer":
            self.form.add_skip(1)
            self.form.add_checkbox(True)  # pick apply for wp with same employer
            self.form.add_skip(1)
            self.form.add_checkbox(True)  # pick restore status
        else:
            if (
                self.applicant.wpincanada.application_purpose
                == "apply WP for same employer"
            ):
                self.form.add_checkbox(True)
                self.form.add_skip(3)
            else:
                self.form.add_skip(1)
            if (
                self.applicant.wpincanada.application_purpose
                == "apply WP for new employer"
            ):
                self.form.add_checkbox(True)
                self.form.add_skip(2)

    def add_work_detail(self):
        """Add details of intended work in Canada"""

        self.form.add_info("intended work section")

        work_detail = self.applicant.wpincanada

        self.form.add_dropdown(
            work_detail.work_permit_type, option_lists.work_permit_ex
        )
        if work_detail.work_permit_type == "Other":
            self.form.add_text(work_detail.other_explain)
        else:
            self.form.add_skip(1)

        if work_detail.employer_name:
            self.form.add_text(work_detail.employer_name)
        else:
            self.form.add_text("Not applicable")

        if work_detail.employer_name:
            self.form.add_text(work_detail.employer_address)
        else:
            self.form.add_text("Not applicable")

        province = work_detail.work_province
        if province:
            self.form.add_dropdown(province, option_lists.canada_province)
        else:
            self.form.add_skip(1)

        if work_detail.work_city:
            self.form.add_dropdown(
                work_detail.work_city.upper(),
                option_lists.canada_province_cities[province],
            )
        else:
            self.form.add_skip(1)

        if work_detail.employer_address:
            self.form.add_text(work_detail.employer_address)
        else:
            self.form.add_text("TO BE DETERMINED")

        if work_detail.job_title:
            self.form.add_text(work_detail.job_title)
        else:
            self.form.add_text("TO BE DETERMINED")

        if work_detail.brief_duties:
            self.form.add_text(work_detail.brief_duties)
        else:
            self.form.add_text("TO BE DETERMINED")

        if work_detail.start_date:
            self.form.add_date(work_detail.start_date)
        else:
            self.form.add_skip(1)

        if work_detail.end_date:
            self.form.add_date(work_detail.end_date)
        else:
            self.form.add_skip(1)

        if work_detail.lmia_num_or_offer_num:
            self.form.add_text(work_detail.lmia_num_or_offer_num)
        else:
            self.form.add_skip(1)

        if work_detail.caq_number:
            self.form.add_text(work_detail.caq_number)
            self.form.add_date(work_detail.expiry_date)
        else:
            self.form.add_skip(2)
        self.form.add_radio(work_detail.pnp_certificated)

    def get_form(self):
        self.start()
        self.add_header()
        self.add_personal_detail()
        self.add_language()
        self.add_passport()
        self.add_national_id()
        self.add_uspr_card()
        self.add_contact_information(form_type="5708")
        self.add_coming_into_canada()
        self.add_work_detail()
        self.add_education()
        self.add_employment()
        self.add_background()
        self.add_signature()
        return self.form



from model.common.contact import Contacts
from webform.webcommon import WebPages

"""
3rd party information and stream determination

"""


class Contact(WebPages):
    @property
    def getPrimaryContact(self):
        return Contacts(self.app.contact).primary.full_name

    @property
    def actions(self):
        return [self.employerContactPage, *self.thirdpartyInfoPages]

    @property
    def employerContactPage(self):
        next_page_tag = "#\\39 130 > div > input:nth-child(1)"  # Yes button for rep

        select_contact = self.web_element.selectElement(
            "#\\39 124 > select",
            self.getPrimaryContact,
            label="Select employer contact",
            select_by_text=True,
        )
        wait1s = self.web_element.waitElement(1000)
        confirm = self.web_element.checkboxElement(
            "#\\39 125 > input", True, label="Confirm it's primary contact"
        )
        add = self.web_element.buttonElement("#addContact", label="Add")
        third_party = self.web_element.radioElement(
            "#\\39 127 > div > input:nth-child(1)", label="Yes"
        )
        page_actions = [select_contact, wait1s, confirm, add, wait1s, third_party]
        return self.web_element.pageElement(
            "#next", next_page_tag, page_actions, label="Employer contact"
        )

    @property
    def thirdpartyInfoPages(self):
        # RCIC name and if paid
        actions1 = [
            self.web_element.selectElement(
                "#\\39 129 > select",
                self.app.rcic.full_name,
                label="Third party",
                select_by_text=True,
            ),
            self.web_element.radioElement(
                "#\\39 130 > div > input:nth-child(1)", label="Is the third party paid?"
            ),
        ]
        page1 = self.web_element.pageElement(
            "#next",
            "#\\39 131 > div > input:nth-child(10)",
            actions=actions1,
            label="Third party representative information",
        )

        # rcic regulator
        actions2 = [
            self.web_element.checkboxElement(
                "#\\39 131 > div > input:nth-child(1)", True, label="ICCRC"
            )
        ]
        page2 = self.web_element.pageElement(
            "#next", "#\\39 132 > input", actions=actions2, label="ICCRC info"
        )

        # RCIC number and LMIA stream
        # EE(Express Entry: 5593),HWS(High Wage Stream: 5626),WS(Low Wage Stream: 5627),
        # GTS(Global Talent Stream: 5624, 5625),AC (Academic: 5626),AG (Agriculture: 5519, 5510), CG(CareGiver 5604)
        stream_ids = {
            "GTS": "#\\39 137 > div > input:nth-child(1)",
            "AG": "#\\39 137 > div > input:nth-child(7)",
            "EE": "#\\39 137 > div > input:nth-child(10)",
            "AC": "#\\39 137 > div > input:nth-child(13)",
            "HWS": "#\\39 137 > div > input:nth-child(16)",
            "LWS": "#\\39 137 > div > input:nth-child(16)",
            "CG": "#\\39 137 > div > input:nth-child(19)",
        }
        actions3 = [
            self.web_element.inputElement(
                "#\\39 132 > input",
                self.app.rcic.rcic_number,
                label="Third party number",
                length=10,
            ),
            self.web_element.radioElement(
                stream_ids.get(self.app.lmiacase.stream_of_lmia),
                label="Stream determination",
            ),
        ]

        # After stream determination
        gts_tag = "#\\39 139 > div > input:nth-child(4)"
        wage_tag = "#\\39 158 > input"
        wage_compare_provincial = "#\\39 138 > div > input:nth-child(1)"
        stream_next_tags = {
            "GTS": gts_tag,
            "AG": wage_tag,
            "EE": wage_tag,
            "AC": wage_tag,
            "HWS": wage_compare_provincial,
            "LWS": wage_compare_provincial,
            "CG": wage_tag,
        }
        page3 = self.web_element.pageElement(
            "#next",
            stream_next_tags.get(self.app.lmiacase.stream_of_lmia),
            actions=actions3,
            label="Third party information and stream determination",
        )

        return [page1, page2, page3]



from datetime import date
from typing import Optional, List
from pydantic import BaseModel, root_validator, EmailStr, validator
from model.common.commonmodel import CommonModel
from model.common.person import Person, PersonalAssess
from model.common.advertisement import (
    Advertisement,
    Advertisements,
    InterviewRecord,
    InterviewRecords,
)
from model.common.address import Address
from model.common.contact import ContactBase
from model.common.jobofferbase import JobofferBase
from model.common.jobposition import PositionBase
from termcolor import colored
from model.common.utils import makeList


class LmiaCase(BaseModel):
    area_index: int
    province_index: int
    unemploy_rate: float
    area_median_wage: float
    noc_outlook: int
    reason_failed_hire_canadian: Optional[str]
    provincial_median_wage: float
    is_in_10_days_priority: Optional[bool]
    top10_wages: float
    is_waived_from_advertisement: bool
    reason_for_waived: Optional[str]
    purpose_of_lmia: str
    stream_of_lmia: str
    has_another_employer: Optional[bool]
    another_employer_name: Optional[str]
    number_of_tfw: int
    duration_number: int
    duration_unit: str
    duration_reason: str
    has_attestation: bool
    use_jobbank: bool
    reason_not_use_jobbank: Optional[str]
    provide_details_even_waived: Optional[str]

    @property
    def duration(self):
        return f"{self.duration_number} {self.duration_unit}"

    @root_validator
    def checkAdditional(cls, values):
        has_another_employer = values.get("has_another_employer", None)
        another_employer_name = values.get("another_employer_name", None)
        if has_another_employer and not another_employer_name:
            raise ValueError(
                "Since has another employer,but did not input the details about it."
            )
        noc_outlook = values.get("noc_outlook", None)
        reason_failed_hire_canadian = values.get("reason_failed_hire_canadian", None)
        if noc_outlook <= 2 and not reason_failed_hire_canadian:
            print(
                colored(
                    "Warning: since the outlook is <= 2, but it seems you didn't consider that you may be challanged by processing officer for why there is lack of labours",
                    "yellow",
                )
            )

        is_waived_from_advertisement = values.get("is_waived_from_advertisement", None)
        reason_for_waived = values.get("reason_for_waived", None)
        provide_details_even_waived = values.get("provide_details_even_waived", None)
        if is_waived_from_advertisement and not reason_for_waived:
            raise ValueError(
                f"Since the occupation is waived from minimum advertisement, but you did not enter the reason for being waived from advertisement."
            )

        if is_waived_from_advertisement and not provide_details_even_waived:
            raise ValueError(
                f"Since the occupation is waived from minimum advertisement, but you did not answer if you would like to provide details about details of recruitment."
            )

        use_jobbank = values.get("use_jobbank", None)
        reason_not_use_jobbank = values.get("reason_not_use_jobbank", None)
        if not use_jobbank and not reason_not_use_jobbank:
            raise ValueError(
                "Since you did not use jobbank,you should provide reason why did not "
            )

        return values


class General(BaseModel):
    legal_name: str
    operating_name: Optional[str]
    website: Optional[str]
    establish_date: date
    company_intro: str
    business_intro: str
    cra_number: str
    ft_employee_number: int
    pt_employee_number: int
    has_lmia_approved: bool
    when_lmia_approved: Optional[date]
    lmia_num: Optional[str]
    recruit_email: EmailStr

    @root_validator
    def checkHasApprovedLmia(cls, values):
        has_lmia_approved = values.get("has_lmia_approved", None)
        when_lmia_approved = values.get("when_lmia_approved", None)
        lmia_num = values.get("lmia_num", None)
        if has_lmia_approved and not (when_lmia_approved or lmia_num):
            raise ValueError(
                "Since has LMIA approved, but no when it got approved or no LMIA number."
            )
        return values

    @property
    def within_2years(self):
        the_day = date.today()
        if self.when_lmia_approved:
            try:
                years = (
                    the_day.year
                    - self.when_lmia_approved.year
                    - (
                        (the_day.month, the_day.day)
                        < (self.when_lmia_approved.month, self.when_lmia_approved.day)
                    )
                )
                return True if years <= 2 else False

            except (ValueError, TypeError) as err:
                print(err.args[0], " in file ", __file__)
        return False


class Lmi(BaseModel):
    laid_off_in_12: bool
    laid_off_canadians: Optional[int]
    laid_off_tfw: Optional[int]
    laid_off_reason: Optional[str]
    is_work_sharing: bool
    work_sharing_info: Optional[str]
    labour_dispute: bool
    labour_dispute_info: Optional[str]
    brief_benefit: str
    job_creation_benefit: Optional[str]
    skill_transfer_benefit: Optional[str]
    fill_shortage_benefit: Optional[str]
    other_benefit: Optional[str]
    canadian_lost_job: bool
    canadian_lost_job_info: Optional[str]
    safety_concerns: str

    @root_validator
    def checkLabourDisput(cls, values):
        canadian_lost_job = values.get("canadian_lost_job", None)
        canadian_lost_job_info = values.get("canadian_lost_job_info", None)
        if canadian_lost_job and not canadian_lost_job_info:
            raise ValueError(
                "Since Canadian will lost job because of hiring TFW,but did not input the details about it."
            )
        return values

    @root_validator
    def checkLayoff(cls, values):
        laid_off_in_12 = values.get("laid_off_in_12", None)
        laid_off_canadians = values.get("laid_off_canadians", None)
        laid_off_tfw = values.get("laid_off_tfw", None)
        laid_off_reason = values.get("laid_off_reason", None)
        if laid_off_in_12 and (
            not laid_off_canadians or not laid_off_tfw or not laid_off_reason
        ):
            raise ValueError(
                "Since there is laid of in past 12 months in info lmi sheet,but did not input how many Canadians and/or foreign workers, and/or reason of lay off."
            )
        return values

    @root_validator
    def checkWorkSharing(cls, values):
        is_work_sharing = values.get("is_work_sharing", None)
        work_sharing_info = values.get("work_sharing_info", None)
        if is_work_sharing and not work_sharing_info:
            raise ValueError(
                "Since there is work sharing in info lmi sheet,but did not input the details about it."
            )
        return values


class ErAddress(Address):
    phone: Optional[str]

    @root_validator
    def checkRowCompletion(cls, values):
        all_fields = [
            "po_box",
            "unit",
            "street_number",
            "street_name",
            "city",
            "district",
            "province",
            "country",
            "post_code",
            "phone",
        ]
        all_fields_values = [values[field] for field in all_fields]

        required_fields = [
            "street_number",
            "street_name",
            "city",
            "country",
            "post_code",
            "phone",
        ]
        variable_type = values.get("variable_type")
        display_type = values.get("display_type")

        required_values = [values[field] for field in required_fields]

        has_values = [value for value in required_values if value]

        if (
            any(all_fields_values)
            and not all(required_values)
            and variable_type != "working_address"
        ):
            raise ValueError(
                f"Please check the row with values ({','.join(has_values)}), some required fileds are missed."
            )

        if (
            any(all_fields_values)
            and not all(required_values)
            and variable_type == "working_address"
            and display_type == "工作地点1"
        ):
            raise ValueError(
                f"Please check the row with values ({','.join(has_values)}), some required fileds are missed."
            )

        return values


class Contact(ContactBase):
    middle_name: Optional[str]
    position: str
    po_box: Optional[str]
    unit: Optional[str]
    street_number: str
    street_name: str
    city: str
    province: str
    post_code: str

    @root_validator
    def checkCanadaProvince(cls, values):
        province = values.get("province")
        if province and province not in [
            "AB",
            "BC",
            "MB",
            "NB",
            "NL",
            "NS",
            "NT",
            "NU",
            "ON",
            "PE",
            "QC",
            "SK",
            "YT",
        ]:
            raise ValueError(
                f'Since country is Canada is, the province must be one of  "AB","BC","MB","NB","NL","NS","NT","NU","ON","PE","QC","SK","YT"'
            )
        return values


class Joboffer(JobofferBase):
    phone_country_code: str
    phone: str
    license_request: bool
    license_description: Optional[str]
    union: bool
    atypical_schedule: bool
    atypical_schedule_explain: Optional[str]
    part_time_explain: Optional[str]
    payment_way: str
    ot_after_hours_unit: Optional[str]
    ot_after_hours: Optional[float]
    is_working: bool
    has_probation: bool
    probation_duration: Optional[str]
    disability_insurance: bool
    dental_insurance: bool
    empolyer_provided_persion: bool
    extended_medical_insurance: bool
    extra_benefits: Optional[str]
    offer_date: date
    supervisor_name: str
    supervisor_title: str
    employer_rep: str
    employer_rep_title: str
    vacation_pay_days: int
    vacation_pay_percentage: float
    duties_brief: str
    duties: list
    english_french: bool
    oral: Optional[str]
    writing: Optional[str]
    reason_for_no: Optional[str]
    other_language_required: bool
    reason_for_other: Optional[str]
    education_level: str
    is_trade: Optional[bool]
    trade_type: Optional[str]
    specific_edu_requirement: Optional[str]
    skill_experience_requirement: Optional[str]
    other_requirements: Optional[list]

    _str2bool_duties = validator("duties", allow_reuse=True, pre=True)(makeList)
    _str2bool_other_requirements = validator(
        "other_requirements", allow_reuse=True, pre=True
    )(makeList)

    @root_validator
    def checkProbation(cls, values):
        has_probation = values.get("has_probation", None)
        probation_duration = values.get("probation_duration")
        if has_probation and not probation_duration:
            raise ValueError(
                "Since has probation is true, but did not input probation duration"
            )
        return values

    @root_validator
    def checkLanguage(cls, values):
        english_french = values.get("english_french", None)
        oral = values.get("oral", None)
        writing = values.get("writing", None)
        reason_for_no = values.get("reason_for_no", None)
        if english_french and (not oral or not writing):
            raise ValueError(
                "Since English or French is true, but did not input oral or/and writting language requirement"
            )
        if not english_french and not reason_for_no:
            raise ValueError(
                "Since there is no English or French requirement, but did not input the reason"
            )
        return values

    @root_validator
    def checkOtherLanguageReason(cls, values):
        other_language_required = values.get("other_language_required", None)
        reason_for_other = values.get("reason_for_other", None)
        if other_language_required and not reason_for_other:
            raise ValueError(
                "Since required other language in job offer sheet,but did not input the reason"
            )
        return values

    @root_validator
    def checkOthers(cls, values):
        atypical_schedule = values.get("atypical_schedule", None)
        atypical_schedule_explain = values.get("atypical_schedule_explain", None)
        if atypical_schedule and not atypical_schedule_explain:
            raise ValueError(
                "Since it is atypical schedule, but you did not provide explaination."
            )

        license_request = values.get("license_request", None)
        license_description = values.get("license_description", None)
        if license_request and not license_description:
            raise ValueError(
                "Since license is required, but you did not provide details."
            )

        days = values.get("days", None)
        hours = values.get("hours", None)
        part_time_explain = values.get("part_time_explain", None)
        if not atypical_schedule and not all([days, hours]):
            raise ValueError(
                "Since it is fixed working time, but you did not provide days per week and hours per week."
            )

        if hours < 30 and not part_time_explain:
            raise ValueError(
                "Since hours per week is less than 30, you did not provide explaination."
            )
        return values


class Personal(Person):
    pass


class Position(PositionBase):
    pass


# RCIC
class Rcic(BaseModel):
    first_name: str
    last_name: str
    rcic_number: str
    email: EmailStr
    company: str
    rcic_rate: str
    how_client_know_rcic: str

    @property
    def full_name(self):
        return self.first_name + " " + self.last_name



from webform.webcommon import WebPages, Page


class HoursPay(WebPages):
    """Hours and Pay. Common module for all streams"""

    @property
    def actions(self):
        return (
            [self.page1, self.page2, self.page3, self.page3_1, self.page4]
            if self.app.joboffer.ot_ratio >= 1.0
            else [self.page1, self.page2, self.page3, self.page4]
        )

    @property
    def page1(self):
        atypical_schedule_id = (
            "#\\39 438 > div > input:nth-child(1)"
            if self.app.joboffer.atypical_schedule
            else "#\\39 438 > div > input:nth-child(4)"
        )
        page_actions = [
            self.web_element.inputElement(
                "#\\39 436 > input",
                str(self.app.joboffer.hours / self.app.joboffer.days),
                label="how many hours per day",
            ),
            self.web_element.inputElement(
                "#\\39 437 > input",
                str(self.app.joboffer.hours),
                label="how many hours per week",
            ),
            self.web_element.radioElement(
                atypical_schedule_id, label="Is atypical schedule?"
            ),
        ]
        next_page_tag = (
            "#\\39 440 > div > input:nth-child(4)"  # job offer full-time-> No
        )
        return Page(
            page_actions, "#next", next_page_tag, label="Page 1 of huors and pay"
        ).page

    @property
    def page2(self):
        is_fulltime_id = (
            "#\\39 440 > div > input:nth-child(1)"
            if self.app.joboffer.is_full_time
            else "#\\39 440 > div > input:nth-child(4)"
        )
        page_actions = (
            [
                self.web_element.areatextElement(
                    "#\\39 439 > textarea",
                    self.app.joboffer.atypical_schedule_explain,
                    label="details of atypical schedule",
                )
            ]
            if not self.app.joboffer.is_full_time
            else []
        )
        page_actions.append(
            self.web_element.radioElement(
                is_fulltime_id, label="is this job offer full time?"
            )
        )
        next_page_tag = "#\\39 442 > div > input:nth-child(4)"
        return Page(
            page_actions, "#next", next_page_tag, label="Page 2 of hours and pay"
        ).page

    @property
    def page3(self):
        has_overtime_pay = True if self.app.joboffer.ot_ratio >= 1.0 else False
        page_actions = (
            [
                self.web_element.areatextElement(
                    "#\\39 441 > textarea",
                    self.app.joboffer.part_time_explain,
                    label="part time job offer explaination",
                )
            ]
            if not self.app.joboffer.is_full_time
            else []
        )
        has_overtime_rate_id = (
            "#\\39 442 > div > input:nth-child(1)"
            if has_overtime_pay
            else "#\\39 442 > div > input:nth-child(4)"
        )
        page_actions.append(
            self.web_element.radioElement(
                has_overtime_rate_id, label="Is there an overtime rate?"
            )
        )
        # if has overtime pay, will has 1 more page for the details, otherwise, go directly to contingent wage page
        next_page_tag = (
            "#\\39 444 > div > input:nth-child(7)"
            if has_overtime_pay
            else "#\\39 449 > div > input:nth-child(4)"
        )
        return Page(
            page_actions, "#next", next_page_tag, label="Page 3 of hours and pay"
        ).page

    @property
    def page3_1(self):
        overtime_determined_ids = {
            "day": "#\\39 444 > div > input:nth-child(1)",
            "week": "#\\39 444 > div > input:nth-child(4)",
            "both": "#\39 444 > div > input:nth-child(7)",
        }
        id = overtime_determined_ids.get(self.app.joboffer.ot_after_hours_unit)
        page_actions = [
            self.web_element.inputElement(
                "#\\39 443 > input",
                str(self.app.joboffer.overtime_rate),
                label="What is the overtime wage?",
            ),
            self.web_element.radioElement(id, label="overtime determined by"),
        ]
        next_page_tag = "#\\39 449 > div > input:nth-child(4)"  # contingent wage No
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Page 3.1: details about overtime wage",
        ).page

    @property
    def page4(self):
        has_overtime_pay = True if self.app.joboffer.ot_ratio >= 1.0 else False
        overtime_hours_ids = {
            "day": "#\\39 445 > input",
            "week": "#\\39 446 > input",
            "both": "#\\39 447 > input",
        }
        id = overtime_hours_ids.get(self.app.joboffer.ot_after_hours_unit)
        page_actions = (
            [
                self.web_element.inputElement(
                    id, str(self.app.joboffer.ot_after_hours), label="how many hours"
                )
            ]
            if has_overtime_pay
            else []
        )

        # We have no contingent wage based on my practice, so here we make it easier by only say no
        page_actions.append(
            self.web_element.radioElement(
                "#\\39 449 > div > input:nth-child(4)",
                label="Answer No for contingent wage",
            )
        )

        next_page_tag = "#\\39 452 > input"
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Page 4 contingent wage in hours and pay",
        ).page



from webform.webcommon import WebPages, Page


class JobOffer5593(WebPages):
    @property
    def actions(self):
        return [self.page1, *self.page2, self.page3, self.page4]

    @property
    def page1(self):
        if self.app.joboffer.english_french:
            radio_id = (
                "#\\39 456 > div > input:nth-child(1)"  # 5593 TODO: 5593 5626的情况不一样
            )
            next_page_tag = (
                "#\\39 461 > div > input:nth-child(4)"  # require other language-> NO id
            )

        elif self.app.joboffer.is_primary_agriculture_position:
            radio_id = "#\\39 456 > div > input:nth-child(4)"
            next_page_tag = (
                "#\\39 463 > div > input:nth-child(4)"  # minimum education -> No id
            )
        else:
            radio_id = "#\\39 456 > div > input:nth-child(7)"
            next_page_tag = (
                "#\\39 463 > div > input:nth-child(4)"  # minimum education -> No id
            )

        page_actions = [
            self.web_element.inputElement(
                "#\\39 452 > input", self.app.joboffer.job_title, label="Job title"
            ),
            self.web_element.areatextElement(
                "#\\39 453 > textarea", self.app.joboffer.duties, label="Job duties"
            ),
            self.web_element.areatextElement(
                "#\\39 454 > textarea",
                self.app.position.why_hire,
                label="The rational for opening this position",
            ),
            self.web_element.inputElement(
                "#\\39 455 > input",
                self.app.joboffer.work_start_date,
                label="Expected employment start date",
                set_value=True,
            ),
            self.web_element.radioElement(
                radio_id,
                label="Require the ability to communicate in a specific language?",
            ),
        ]

        return Page(
            page_actions, "#next", next_page_tag, label="Page 1 of job offer"
        ).page

    @property
    def page2(self):
        # has minimum edu required
        minimum_edu_req_id = (
            "#\\39 463 > div > input:nth-child(1)"
            if self.app.joboffer.education_level
            else "#\\39 463 > div > input:nth-child(4)"
        )
        minimum_edu_req = self.web_element.radioElement(
            minimum_edu_req_id, label="Minimum education reuqired"
        )

        next_page_tag = "#\\39 467 > div > input:nth-child(4)"  # Is the occupation regulated -> No id
        # if required English/French
        if self.app.joboffer.english_french:
            oral_ids = {
                "English": "#\\39 459 > div > input:nth-child(1)",
                "French": "#\\39 459 > div > input:nth-child(4)",
                "English and French": "#\\39 459 > div > input:nth-child(7)",
                "English or French": "#\\39 459 > div > input:nth-child(10)",
            }
            writting_ids = {
                "English": "#\\39 460 > div > input:nth-child(1)",
                "French": "#\\39 460 > div > input:nth-child(4)",
                "English and French": "#\\39 460 > div > input:nth-child(7)",
                "English or French": "#\\39 460 > div > input:nth-child(10)",
            }
            other_language_id = (
                "#\\39 461 > div > input:nth-child(1)"
                if self.app.joboffer.other_language_required
                else "#\\39 461 > div > input:nth-child(4)"
            )
            oral = self.web_element.radioElement(
                oral_ids.get(self.app.joboffer.oral), label="Oral"
            )
            writting = self.web_element.radioElement(
                writting_ids.get(self.app.joboffer.writing), label="Writting"
            )
            other_language = self.web_element.radioElement(
                other_language_id, label="Require other language than English/French"
            )
            next_page_tag = (
                "#\\39 463 > div > input:nth-child(4)"  # minumum edu -> NO id
            )
            # page 1 of yes, input details
            p1 = Page(
                [oral, writting, other_language],
                "#next",
                next_page_tag,
                label="Page 2 Language English/French detail",
            ).page

            p2_actions = (
                [
                    self.web_element.areatextElement(
                        "#\\39 462 > textarea",
                        self.app.joboffer.reason_for_other,
                        label="Reason for other language required",
                    )
                ]
                if self.app.joboffer.other_language_required
                else []
            )
            p2_actions.append(minimum_edu_req)
            p2 = Page(
                p2_actions,
                "#next",
                next_page_tag,
                label="Page 2:has language required of Job offer detail",
            ).page

            return [p1, p2]

        elif self.app.joboffer.is_primary_agriculture_position:
            return [
                Page(
                    [minimum_edu_req],
                    "#next",
                    next_page_tag,
                    label="Page 2: language exempted for primary agriculture",
                ).page
            ]
        else:
            # if not required English/French
            rationale = self.web_element.areatextElement(
                "#\\39 457 > textarea",
                self.app.joboffer.reason_for_no,
                label="Rationale for no English/French requirement",
            )
            return [
                Page(
                    [rationale, minimum_edu_req],
                    "#next",
                    next_page_tag,
                    label="Page 2: no language required",
                ).page
            ]

    @property
    def page3(self):
        # education

        if self.app.joboffer.is_trade:
            trade_types = {
                "Apprenticeship diploma/certificate": "#\\39 464 > div > input:nth-child(1)",
                "Trade diploma/certificate": "#\\39 464 > div > input:nth-child(34)",
                "Vocational school diploma/certificate": "#\\39 464 > div > input:nth-child(37)",
            }
            minimum_edu_id = trade_types.get(self.app.joboffer.trade_type)
        else:
            minimum_edu_ids = {
                "Doctor": "#\\39 464 > div > label:nth-child(14)",
                "Master": "#\\39 464 > div > input:nth-child(16)",
                "Post-graduate diploma": None,
                "Bachelor": "#\\39 464 > div > input:nth-child(4)",
                "Associate": None,
                "Diploma/Certificate": "#\\39 464 > div > input:nth-child(7)",
                "High school": "#\\39 464 > div > input:nth-child(31)",
                "Less than high school": "#\\39 464 > div > input:nth-child(19)",
            }
            minimum_edu_id = minimum_edu_ids.get(self.app.joboffer.education_level)
        minimum_edu = self.web_element.radioElement(
            minimum_edu_id, label="Indicate the minimum education"
        )

        specific_edu = self.web_element.areatextElement(
            "#\\39 465 > textarea",
            self.app.joboffer.specific_edu_requirement,
            label="Specific education requirement",
        )
        experience = self.web_element.areatextElement(
            "#\\39 466 > textarea",
            self.app.joboffer.skill_experience_requirement,
            label="Experience requirement",
        )
        license_required_id = (
            "#\\39 467 > div > input:nth-child(1)"
            if self.app.joboffer.license_request
            else "#\\39 467 > div > input:nth-child(4)"
        )
        license_required = self.web_element.radioElement(
            license_required_id, label="Is the occupation regulated?"
        )
        page_actions = [minimum_edu, specific_edu, experience, license_required]
        next_page_tag = "#\\39 470 > div > input:nth-child(4)"  # is the position part of a union -> No id
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Page 3: Education of job offer details",
        ).page

    @property
    def page4(self):
        page_actions = (
            [
                self.web_element.areatextElement(
                    "#\39 468 > textarea",
                    self.app.joboffer.license_description,
                    label="License body",
                )
            ]
            if self.app.joboffer.license_request
            else []
        )

        union_id = (
            "#\\39 470 > div > input:nth-child(1)"
            if self.app.joboffer.union
            else "#\\39 470 > div > input:nth-child(4)"
        )
        page_actions.append(
            self.web_element.radioElement(
                union_id, label="Is the position part of a union?"
            )
        )
        next_page_tag = (
            "#\\39 471 > div > input:nth-child(4)"  # will you provide benefits -> No id
        )
        return Page(page_actions, "#next", next_page_tag, label="Union").page


class JobOffer(WebPages):
    @property
    def actions(self):
        return [self.page1, *self.page2, self.page3, self.page4]

    @property
    def page1(self):
        if self.app.joboffer.english_french:
            radio_id = "#\\39 456 > div > input:nth-child(1)"
            next_page_tag = (
                "#\\39 461 > div > input:nth-child(4)"  # require other language-> NO id
            )

        elif self.app.joboffer.is_primary_agriculture_position:
            radio_id = "#\\39 456 > div > input:nth-child(4)"
            next_page_tag = (
                "#\\39 463 > div > input:nth-child(4)"  # minimum education -> No id
            )
        else:
            radio_id = "#\\39 456 > div > input:nth-child(7)"
            next_page_tag = (
                "#\\39 463 > div > input:nth-child(4)"  # minimum education -> No id
            )

        page_actions = [
            self.web_element.inputElement(
                "#\\39 452 > input", self.app.joboffer.job_title, label="Job title"
            ),
            self.web_element.areatextElement(
                "#\\39 453 > textarea", self.app.joboffer.duties, label="Job duties"
            ),
            self.web_element.areatextElement(
                "#\\39 454 > textarea",
                self.app.position.why_hire,
                label="The rational for opening this position",
            ),
            self.web_element.inputElement(
                "#\\39 455 > input",
                self.app.joboffer.work_start_date,
                label="Expected employment start date",
                set_value=True,
            ),
            self.web_element.radioElement(
                radio_id,
                label="Require the ability to communicate in a specific language?",
            ),
        ]

        return Page(
            page_actions, "#next", next_page_tag, label="Page 1 of job offer"
        ).page

    @property
    def page2(self):
        # has minimum edu required
        minimum_edu_req_id = (
            "#\\39 463 > div > input:nth-child(1)"
            if self.app.joboffer.education_level
            else "#\\39 463 > div > input:nth-child(4)"
        )
        minimum_edu_req = self.web_element.radioElement(
            minimum_edu_req_id, label="Minimum education reuqired"
        )

        next_page_tag = "#\\39 467 > div > input:nth-child(4)"  # Is the occupation regulated -> No id
        # if required English/French
        if self.app.joboffer.english_french:
            oral_ids = {
                "English": "#\\39 459 > div > input:nth-child(1)",
                "French": "#\\39 459 > div > input:nth-child(4)",
                "English and French": "#\\39 459 > div > input:nth-child(7)",
                "English or French": "#\\39 459 > div > input:nth-child(10)",
            }
            writting_ids = {
                "English": "#\\39 460 > div > input:nth-child(1)",
                "French": "#\\39 460 > div > input:nth-child(4)",
                "English and French": "#\\39 460 > div > input:nth-child(7)",
                "English or French": "#\\39 460 > div > input:nth-child(10)",
            }
            other_language_id = (
                "#\\39 461 > div > input:nth-child(1)"
                if self.app.joboffer.other_language_required
                else "#\\39 461 > div > input:nth-child(4)"
            )
            oral = self.web_element.radioElement(
                oral_ids.get(self.app.joboffer.oral), label="Oral"
            )
            writting = self.web_element.radioElement(
                writting_ids.get(self.app.joboffer.writing), label="Writting"
            )
            other_language = self.web_element.radioElement(
                other_language_id, label="Require other language than English/French"
            )
            next_page_tag = (
                "#\\39 463 > div > input:nth-child(4)"  # minumum edu -> NO id
            )
            # page 1 of yes, input details
            p1 = Page(
                [oral, writting, other_language],
                "#next",
                next_page_tag,
                label="Page 2 Language English/French detail",
            ).page

            p2_actions = (
                [
                    self.web_element.areatextElement(
                        "#\\39 462 > textarea",
                        self.app.joboffer.reason_for_other,
                        label="Reason for other language required",
                    )
                ]
                if self.app.joboffer.other_language_required
                else []
            )
            p2_actions.append(minimum_edu_req)
            p2 = Page(
                p2_actions,
                "#next",
                next_page_tag,
                label="Page 2:has language required of Job offer detail",
            ).page

            return [p1, p2]

        elif self.app.joboffer.is_primary_agriculture_position:
            return [
                Page(
                    [minimum_edu_req],
                    "#next",
                    next_page_tag,
                    label="Page 2: language exempted for primary agriculture",
                ).page
            ]
        else:
            # if not required English/French
            rationale = self.web_element.areatextElement(
                "#\\39 457 > textarea",
                self.app.joboffer.reason_for_no,
                label="Rationale for no English/French requirement",
            )
            return [
                Page(
                    [rationale, minimum_edu_req],
                    "#next",
                    next_page_tag,
                    label="Page 2: no language required",
                ).page
            ]

    @property
    def page3(self):
        # education

        if self.app.joboffer.is_trade:
            trade_types = {
                "Apprenticeship diploma/certificate": "#\\39 464 > div > input:nth-child(1)",
                "Trade diploma/certificate": "#\\39 464 > div > input:nth-child(34)",
                "Vocational school diploma/certificate": "#\\39 464 > div > input:nth-child(37)",
            }
            minimum_edu_id = trade_types.get(self.app.joboffer.trade_type)
        else:
            minimum_edu_ids = {
                "Doctor": "#\\39 464 > div > label:nth-child(14)",
                "Master": "#\\39 464 > div > input:nth-child(16)",
                "Post-graduate diploma": None,
                "Bachelor": "#\\39 464 > div > input:nth-child(4)",
                "Associate": None,
                "Diploma/Certificate": "#\\39 464 > div > input:nth-child(7)",
                "High school": "#\\39 464 > div > input:nth-child(31)",
                "Less than high school": "#\\39 464 > div > input:nth-child(19)",
            }
            minimum_edu_id = minimum_edu_ids.get(self.app.joboffer.education_level)
        minimum_edu = self.web_element.radioElement(
            minimum_edu_id, label="Indicate the minimum education"
        )

        specific_edu = self.web_element.areatextElement(
            "#\\39 465 > textarea",
            self.app.joboffer.specific_edu_requirement,
            label="Specific education requirement",
        )
        experience = self.web_element.areatextElement(
            "#\\39 466 > textarea",
            self.app.joboffer.skill_experience_requirement,
            label="Experience requirement",
        )
        license_required_id = (
            "#\\39 467 > div > input:nth-child(1)"
            if self.app.joboffer.license_request
            else "#\\39 467 > div > input:nth-child(4)"
        )
        license_required = self.web_element.radioElement(
            license_required_id, label="Is the occupation regulated?"
        )
        page_actions = [minimum_edu, specific_edu, experience, license_required]
        next_page_tag = "#\\39 470 > div > input:nth-child(4)"  # is the position part of a union -> No id
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Page 3: Education of job offer details",
        ).page

    @property
    def page4(self):
        page_actions = (
            [
                self.web_element.areatextElement(
                    "#\39 468 > textarea",
                    self.app.joboffer.license_description,
                    label="License body",
                )
            ]
            if self.app.joboffer.license_request
            else []
        )

        union_id = (
            "#\\39 470 > div > input:nth-child(1)"
            if self.app.joboffer.union
            else "#\\39 470 > div > input:nth-child(4)"
        )
        page_actions.append(
            self.web_element.radioElement(
                union_id, label="Is the position part of a union?"
            )
        )
        next_page_tag = (
            "#\\39 471 > div > input:nth-child(4)"  # will you provide benefits -> No id
        )
        return Page(page_actions, "#next", next_page_tag, label="Union").page



import json, os, shutil
from mailbox import MMDFMessage
import time
import subprocess

import context
from pdfform import config_mac as config
from pdfform.application_form_mac import ApplicationForm


# Get project's home directory,
BASEDIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
# All data directory
DATADIR = os.path.abspath(os.path.join(BASEDIR, "data"))


def check_env():
    # if not os.path.isfile(config.ACROBAT_READER_PATH):
    #     print("Acrobat reader not found. Please install it first.")
    #     exit()
    if not os.path.isdir(config.OUTPUT_PATH):
        print("Output directory doesn't exist. Please check.")
        exit()


def prepare_document():
    """Create a new empty pdf file to fill"""

    # copy empty template file to output folder
    # src = DATADIR + f"/pdf/imm5708.pdf"
    dst = config.OUTPUT_PATH + "imm1295e.pdf"
    # shutil.copy2(src, dst)

    subprocess.call(["/usr/bin/open", dst])
    time.sleep(3)  # wait 5 seconds for file open finished.


def main():
    check_env()
    prepare_document()
    with open("/Users/jacky/desktop/1295.json") as f:
        actions = json.load(f)
    form = ApplicationForm(None)
    for action in actions:
        form.add_step(action)
    form.fill_form(start=0, verbose=True)


if __name__ == "__main__":
    main()



from pathlib import Path
from model.experience.data import (
    Employment,
    Personal,
    Language,
    PersonalAssess,
    Education,
)
from model.common.employmenthistory import EmploymentHistory
from model.common.id import ID, IDs
from typing import List, Optional
from model.common.commonmodel import CommonModel, BuilderModel
from model.common.phone import Phone, Phones
from model.common.address import Address, Addresses
from pydantic import BaseModel


class PersonId(ID):
    pass


DATADIR = Path(__file__).parents[2] / "data"


class ExperienceModel(BaseModel, BuilderModel):
    personal: Personal
    phone: List[Phone]
    personalassess: PersonalAssess
    education: List[Education]
    language: Optional[List[Language]]
    employment: Optional[List[Employment]]
    address: List[Address]
    personid: List[PersonId]

    def context(self, doc_type=None):
        if doc_type == "ec":
            # Get company set
            try:
                company_set = set(
                    [
                        emp.company
                        for emp in self.employment
                        if emp.employment_certificate
                    ]
                )
            except KeyError as err:
                raise ValueError(
                    f'key error in employment sheet. Please check "company","employment","employment_certificate"'
                )
            # pick a company for employment certificate generation
            companies = list(company_set)
            [print(index, company) for index, company in enumerate(companies)]
            which_one = input("Please input the number of the employer: ")
            company = companies[int(which_one)]

            # get validated data
            m_ec = []
            is_current = False
            works = [emp for emp in self.employment if emp.company == company]

            for i, employment in enumerate(works):
                if not employment.end_date:
                    is_current = True
                m_ec.append(employment)
            # get summary info
            history = EmploymentHistory(m_ec)
            start_date = history.initial_start_date
            end_date = history.final_end_date
            position_number = history.position_number_say
            ids = IDs(self.personid)

            # get company brief, and hr rep's information from the list of positions in one company. The rep info could be in any period, but only get one of them.
            def getContent(field):
                for w in m_ec:
                    value = getattr(w, field)
                    if value:
                        return value
                return None

            context = {
                # 'company_brief':company_brief,
                "company_brief": getContent("company_brief"),
                "hr_rep_name": getContent("fullname_of_certificate_provider"),
                "hr_rep_position": getContent("position_of_certificate_provider"),
                "hr_rep_department": getContent("department_of_certificate_provider"),
                "hr_rep_phone": getContent("phone_of_certificate_provider"),
                "hr_rep_email": getContent("email_of_certificate_provider"),
                "passport": ids.passport.number,
                "work": m_ec,
                "personal": self.personal,
                "more_than_1": len(m_ec) > 1,
                "position_number": position_number,
                "is_current": is_current,
                "start_date": start_date.strftime("%b %Y"),
                "end_date": end_date,
            }
        elif doc_type == "rs":
            for emp in self.employment:
                emp.start_date = emp.start_from
                emp.end_date = emp.end_to
            for edu in self.education:
                edu.start_date = edu.start_from
                edu.end_date = edu.end_to
            context = {
                **self.dict(),
                "phone": Phones(self.phone).PreferredPhone.international_format_full,
                "address": Addresses(self.address).PreferredAddress.full_address,
                "full_name": self.personal.full_name,
            }
        else:
            context = {}

        return context

    def make_web_form(self, output_json, upload_dir, rcic):
        pass

    def make_pdf_form(self):
        pass


class ExperienceModelE(CommonModel, ExperienceModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/pa.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from typing import List
from model.common.commonmodel import CommonModel, BuilderModel
from model.recruit.jobaddata import JobOffer, General, ErAddress, ErAddresses
from model.common.wordmaker import WordMaker
import os
from pydantic import BaseModel


class JobadModel(BaseModel, BuilderModel):
    general: General
    joboffer: JobOffer
    eraddress: List[ErAddress]

    def context(self, *args, **kwargs):
        context = {
            **self.dict(),
            "workingaddress": ErAddresses(self.eraddress).working.full_address,
            "term": self.joboffer.term,
            "weekly_hours": self.joboffer.weekly_hours,
            "salary": self.joboffer.salary,
            "wage_unit": self.joboffer.wage_unit,
            "benefit": self.joboffer.benefits,
        }
        return context

    def make_pdf_form(self, *args, **kwargs):
        pass

    def make_web_form(self, *args, **kwargs):
        pass


class JobadModelE(CommonModel, JobadModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/er.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())


class JobadModelDocxAdapater:
    """This is an adapater to bridging job ad model data and docx data"""

    def __init__(self, jobad_obj: JobadModel):
        # get original obj, which will be used to generate some value based on it's object methods.
        # 此处用来处理list里面的一些内容。
        self.jobad_obj = jobad_obj
        addresses = ErAddresses(self.jobad_obj.eraddress)
        self.jobad_obj.eraddress = addresses.working

    def make(self, output_docx, template_no=None):
        template_path = os.path.abspath(os.path.join(DATADIR, "word/jobad.docx"))
        wm = WordMaker(template_path, self.jobad_obj, output_docx)
        wm.make()



import context
from datetime import date
import requests
import json
from source.excel import Excel
from pprint import pprint


def getJsonData(excel_file_name):
    e = Excel(excel_name=excel_file_name)
    return json.loads(e.json)


headers = {"accept": "application/json", "Content-Type": "application/json"}

# # make
# m5593 = {
#     "command": "make",
#     "model":"lmia-5593",
#     "output_name":"5593.xlsx",
# }

# fn=m5593['output_name']
# r = requests.post("http://localhost:8000/imm", json=m5593)

# with open(fn, 'wb') as f:
#     f.write(r.content)
# print(f"{fn} has been downloaded from web")

# # check
# m5593 = {
#     "command": "check",
#     "model":"lmia-5593",
#     "data":getJsonData("/Users/jacky/desktop/demo/5593.xlsx"),
#     # "output_name":"5593.xlsx",
# }
# r = requests.post("http://localhost:8000/imm", json=m5593)
# if r.status_code==200:
#     print(r.json()["success"])
# elif r.status_code==422:
#     print(r.json()['detail'])

# # word
# m5593={
#     "command": "word",
#     "model": "lmia-5593",
#     "data":getJsonData("/Users/jacky/desktop/demo/5593.xlsx"),
#     "output_name": "sl.docx",
#     "print_json": True,
#     "doctype": "sl",
#     "rciccompany": "noah",
#     "tempnum": 1,
# }
# r = requests.post("http://localhost:8000/imm", json=m5593)
# fn=m5593['output_name']
# with open(fn, 'wb') as f:
#     f.write(r.content)
# print(f"{fn} has been downloaded from web")


# #webform
# m5593={
#     "command": "webform",
#     "model": "lmia-5593",
#     "data":getJsonData("/Users/jacky/desktop/demo/5593.xlsx"),
#     "output_name": "5593.json",
#     "rcic": "jacky",
#     "upload_dir":"."
# }
# r = requests.post("http://localhost:8000/imm", json=m5593)
# fn=m5593['output_name']
# with open(fn, 'wb') as f:
#     f.write(r.content)
# print(f"{fn} has been downloaded from web")

# # pdfform
# m5709 = {
#     "command": "pdfform",
#     "model": "5709",
#     "data": getJsonData("/Users/jacky/desktop/demo/5709.xlsx"),
#     "output_name": "5709.json",
#     "rcic": "jacky",
#     "upload_dir": ".",
# }
# r = requests.post("http://localhost:8000/imm", json=m5709)
# fn = m5709["output_name"]
# with open(fn, "wb") as f:
#     f.write(r.content)
# print(f"{fn} has been downloaded from web")

# cap={
#     "command": "run",
#     "model": "lmia-cap",
#     "data":getJsonData("/Users/jacky/desktop/demo/cap.xlsx"),
# }
# r = requests.post("http://localhost:8000/imm", json=cap)

# if r.status_code==200:
#     print(r.json()["success"])
# elif r.status_code==422:
#     print(r.json()['detail'])


# get user
r = requests.get("http://localhost:8000/user")
pprint(r.json())

# # read
# r = requests.get("http://localhost:8000/user/630aa43fb5e1ee46a63716f6")
# print(r.status_code, r.content)

# create user
# jacky = {
#     "username": "may",
#     "email": "jackyzhang1969@gmail.com",
#     "phone": "7783215110",
#     "password": "555",
# }
# r = requests.post("http://localhost:8000/user/", json=jacky)
# print(r.status_code, r.content)

# update
# jacky = {
#     "username": "lele",
#     "email": "jackyzhang1969@gmail.com",
#     "phone": "7783215110",
#     "password": "xxxxx",
# }
# r = requests.put("http://localhost:8000/user/630aa056f0e34b68a8d7b9bf", json=jacky)
# print(r.status_code, r.content)

# # delete
# r = requests.delete("http://localhost:8000/user/630aa056f0e34b68a8d7b9bf")
# print(r.status_code, r.content)



from re import L
from turtle import position
from webform.webcommon import WebPages, Page

# 1. Cap for LWS
class Cap(WebPages):
    @property
    def actions(self):
        return [*self.page1s]

    @property
    def page1s(self):
        has_same_position_id = (
            "#\\39 433 > div > input:nth-child(1)"
            if self.app.position.has_same
            else "#\\39 433 > div > input:nth-child(4)"
        )
        has_same_position = self.web_element.radioElement(
            has_same_position_id, label="Yes" if self.app.position.has_same else "No"
        )
        if self.app.emp5627.cap_exempted:
            which_exemption_action = self.which_exemption
            rationale = self.web_element.areatextElement(
                "#\\39 393 > textarea", self.app.emp5627.exemption_rationale
            )
            next_page_tag = "#\\39 438 > div > input:nth-child(4)"

            return [
                Page(
                    [which_exemption_action, rationale, has_same_position],
                    "#next",
                    next_page_tag,
                    label="CAP exemption yes",
                ).page
            ]
        else:
            in_seasonal_industry = self.inSeasonalIndustry
            is_in = self.app.emp5627.is_in_seasonal_industry
            if is_in:
                return [in_seasonal_industry, self.wish_to_go1, self.page2]
            else:
                return [in_seasonal_industry, self.wish_to_go2, self.page2]

    @property
    def page2(self):
        has_someone_in_same_position_id = (
            "#\\39 433 > div > input:nth-child(1)"
            if self.app.position.has_same
            else "#\\39 433 > div > input:nth-child(4)"
        )
        has_someone_in_same_position = self.web_element.radioElement(
            has_someone_in_same_position_id,
            label="Yes" if self.app.position.has_same else "No",
        )
        next_page_tag = "#\\39 438 > div > input:nth-child(4)"  # atypical No id
        return self.web_element.pageElement(
            "#next",
            next_page_tag,
            [has_someone_in_same_position],
            label="Has same position?",
        )

    @property
    def which_exemption(self):
        exemption_types = {
            "Caregiver positions in a health care facility (NOC 3012, 3233, and 3413)": "#\\39 392 > div > input:nth-child(1)",
            "On-farm primary agricultural positions": "#\\39 392 > div > input:nth-child(4)",
            "Position for the Global Talent Stream": "#\\39 392 > div > input:nth-child(7)",
            "Position(s) is/are highly mobile": "#\\39 392 > div > input:nth-child(10)",
            "Position(s) is/are truly temporary": "#\\39 392 > div > input:nth-child(13)",
            "Seasonal 270-day exemption": "#\\39 392 > div > input:nth-child(16)",
        }
        exemption_id = exemption_types.get(self.app.emp5627.which_exemption)
        return self.web_element.radioElement(
            exemption_id, label=self.app.emp5627.which_exemption
        )

    @property
    def inSeasonalIndustry(self):
        is_in = self.app.emp5627.is_in_seasonal_industry
        is_in_id = (
            "#\\39 395 > div > input:nth-child(1)"
            if is_in
            else "#\\39 395 > div > input:nth-child(4)"
        )
        is_in_action = self.web_element.radioElement(
            is_in_id, label="Yes" if is_in else "No"
        )
        # if yes in, next page tag is: do you wish to continue, no id
        # if no, do you wish to continue. no id, but different value
        next_page_tag = (
            "#\\39 429 > div > input:nth-child(4)"
            if is_in
            else "#\\39 411 > div > input:nth-child(4)"
        )
        return Page(
            [is_in_action],
            "#next",
            next_page_tag,
            label="low wage, is in seasonal page?",
        ).page

    @property
    def wish_to_go1(self):
        # answer the question always as Yes
        wish_to_go_id = "#\\39 429 > div > input:nth-child(1)"
        to_go = self.web_element.radioElement(wish_to_go_id, label="Yes")

        # CAP calculation
        add = self.web_element.buttonElement("#addSeasonal", label="Add")
        wait_for = self.web_element.waitForElement("#\\39 420 > select")
        # use first word of employer name as keyword to pick
        employer_name_keyword = self.app.general.legal_name.split(" ")[0]
        employer = self.web_element.selectElement(
            "#\\39 420 > select",
            employer_name_keyword,
            label="pick employer",
            select_by_text=True,
        )
        from_date = self.web_element.inputElement(
            "#\\39 421 > input",
            self.app.emp5627.four_week_start_date,
            label="From",
            set_value=True,
        )
        to_date = self.web_element.inputElement(
            "#\\39 422 > input",
            self.app.emp5627.four_week_end_date,
            label="To",
            set_value=True,
        )
        q_a = self.web_element.inputElement(
            "#\\39 423 > input", str(self.app.emp5627.q_a), label="Question A"
        )
        q_b = self.web_element.inputElement(
            "#\\39 424 > input", str(self.app.emp5627.q_b), label="Question B"
        )
        q_c = self.web_element.inputElement(
            "#\\39 425 > input", str(self.app.emp5627.q_c), label="Question C"
        )
        q_d = self.web_element.inputElement(
            "#\\39 426 > input", str(self.app.emp5627.q_d), label="Question D"
        )
        q_e = self.web_element.inputElement(
            "#\\39 427 > input", str(self.app.emp5627.q_e), label="Question E"
        )
        q_f = self.web_element.inputElement(
            "#\\39 417 > input", str(self.app.emp5627.q_f), label="Question F"
        )
        q_g = self.web_element.inputElement(
            "#\\39 418 > input", str(self.app.emp5627.q_g), label="Question G"
        )
        q_h = self.web_element.inputElement(
            "#\\39 419 > input", str(self.app.emp5627.q_h), label="Question H"
        )
        save = self.web_element.buttonElement("#saveSeasonal", label="Save")
        # wait_for_after_save = self.web_element.waitForElement(
        # "#seasonalTable > tbody > tr > td:nth-child(23) > button.btn.btn-default.editSeasonal"
        # )  # wait for table's edit button
        wait_for_after_save = self.web_element.waitElement(1000)
        page_actions = [
            to_go,
            add,
            wait_for,
            employer,
            from_date,
            to_date,
            q_a,
            q_b,
            q_c,
            q_d,
            q_e,
            q_f,
            q_g,
            q_h,
            save,
            wait_for_after_save,
        ]
        # return page
        next_page_tag = "#\\39 433 > div > input:nth-child(4)"  # Hours and pay No id
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Wish to go and CAP calculation?",
        ).page

    @property
    def wish_to_go2(self):
        # answer the question always as Yes
        wish_to_go_id = "#\\39 411 > div > input:nth-child(1)"
        to_go = self.web_element.radioElement(wish_to_go_id, label="Yes")

        # CAP calculation
        add = self.web_element.buttonElement("#addImpactcap", label="Add")
        wait_for = self.web_element.waitForElement("#\\39 420 > select")
        # use first word of employer name as keyword to pick
        employer_name_keyword = self.app.general.legal_name.split(" ")[0]
        employer = self.web_element.selectElement(
            "#\\39 399 > select",
            employer_name_keyword,
            label="pick employer",
            select_by_text=True,
        )
        from_date = self.web_element.inputElement(
            "#\\39 400 > input",
            self.app.emp5627.four_week_start_date,
            label="From",
            set_value=True,
        )
        to_date = self.web_element.inputElement(
            "#\\39 401 > input",
            self.app.emp5627.four_week_end_date,
            label="To",
            set_value=True,
        )
        q_a = self.web_element.inputElement(
            "#\\39 402 > input", str(self.app.emp5627.q_a), label="Question A"
        )
        q_b = self.web_element.inputElement(
            "#\\39 403 > input", str(self.app.emp5627.q_b), label="Question B"
        )
        q_c = self.web_element.inputElement(
            "#\\39 404 > input", str(self.app.emp5627.q_c), label="Question C"
        )
        q_d = self.web_element.inputElement(
            "#\\39 405 > input", str(self.app.emp5627.q_d), label="Question D"
        )
        q_e = self.web_element.inputElement(
            "#\\39 406 > input", str(self.app.emp5627.q_e), label="Question E"
        )
        q_f = self.web_element.inputElement(
            "#\\39 407 > input", str(self.app.emp5627.q_f), label="Question F"
        )
        q_g = self.web_element.inputElement(
            "#\\39 408 > input", str(self.app.emp5627.q_g), label="Question G"
        )
        q_h = self.web_element.inputElement(
            "#\\39 409 > input", str(self.app.emp5627.q_h), label="Question H"
        )
        save = self.web_element.buttonElement("#saveImpactcap", label="Save")
        # wait_for_after_save = self.web_element.waitForElement(
        #     "#impactcapTable > tbody > tr > td:nth-child(23) > button.btn.btn-default.editImpactcap"
        # )  # wait for table's edit button
        wait_for_after_save = self.web_element.waitElement(1000)

        page_actions = [
            to_go,
            add,
            wait_for,
            employer,
            from_date,
            to_date,
            q_a,
            q_b,
            q_c,
            q_d,
            q_e,
            q_f,
            q_g,
            q_h,
            save,
            wait_for_after_save,
        ]
        # return page
        next_page_tag = "#\\39 433 > div > input:nth-child(4)"  # Hours and pay No id
        return Page(
            page_actions,
            "#next",
            next_page_tag,
            label="Wish to go and CAP calculation?",
        ).page



from model.common.address import Addresses
from webform.webcommon import WebPages

from typing import Union

# 1. EE
class Wage(WebPages):
    @property
    def explain_calculation(self):
        return self.app.joboffer.how_to_convert_to_hourly_rate

    @property
    def actions(self):
        return [self.wagePage1, self.wagePage2]

    # hourly rate
    @property
    def wagePage1(self):
        page_actions = [
            self.web_element.inputElement(
                "#\\39 158 > input", self.app.joboffer.hourly_rate, label="hourly rate"
            ),
            self.web_element.radioElement(
                "#\\39 159 > div > input:nth-child(1)"
                if self.app.joboffer.wage_unit != "hourly"
                else "#\\39 159 > div > input:nth-child(4)",
                label="Yes" if self.app.joboffer.wage_unit != "hourly" else "No",
            ),
        ]
        next_page_tag = "#\\39 175 > select"  # Duration selector
        return self.web_element.pageElement(
            "#next", next_page_tag, page_actions, label="Wage hourly rate"
        )

    # Wage details about how to calculate and Duration
    @property
    def wagePage2(self):
        # explain how to convert the hourly rate if it's converted from other way
        is_hourly_rate_converted = (
            True if self.app.joboffer.wage_unit != "hourly" else False
        )
        explain_calculation = (
            self.web_element.areatextElement(
                "#\\39 160 > textarea", self.explain_calculation
            )
            if is_hourly_rate_converted
            else None
        )

        duration_justification = self.web_element.areatextElement(
            "#\\39 176 > textarea",
            self.app.lmiacase.duration_reason,
            label="Duration justification",
        )
        number_of_tfw = self.web_element.inputElement(
            "#\\39 177 > input", str(self.app.lmiacase.number_of_tfw)
        )
        duration_number = self.web_element.inputElement(
            "#\\39 174 > input",
            str(self.app.lmiacase.duration_number),
            label="Employment duration",
        )
        duration_unit_types = {
            "years": "Y",
            "months": "M",
            "weeks": "W",
            "days": "D",
            "permanent": "P",
        }
        duration_unit = self.web_element.selectElement(
            "#\\39 175 > select",
            duration_unit_types.get(self.app.lmiacase.duration_unit),
            label="Employment duration unit",
        )

        page_actions = []

        if is_hourly_rate_converted:
            page_actions.append(explain_calculation)

        page_actions += [
            *self.work_location,
            duration_justification,
            number_of_tfw,
            duration_number,
            duration_unit,
        ]
        match self.app.lmiacase.stream_of_lmia:
            case "EE":
                next_page_tag = "#\\39 181 > div > input:nth-child(1)"  # Yes
            case "LWS" | "HWS":
                next_page_tag = (
                    "#\\39 180 > div > input:nth-child(4)"  # provide name: No id
                )
            case _:
                next_page_tag = "#\\39 181 > div > input:nth-child(1)"  # Yes

        return self.web_element.pageElement(
            "#next", next_page_tag, page_actions, label="Wage hourly rate"
        )

    @property
    def work_location(self):
        locations_actions = []

        work_locations = Addresses(self.app.eraddress).workings or []
        business_location = Addresses(self.app.eraddress).business

        for location in work_locations:
            add = self.web_element.buttonElement(
                "#addWorkloctn", label="Add a work location"
            )
            # wait until the checkbox appears
            wait = self.web_element.waitForElement("#\\39 171 > input")
            operating_name = self.web_element.inputElement(
                "#\\39 163 > input",
                self.app.general.operating_name
                if self.app.general.operating_name
                else self.app.general.legal_name,
                length=200,
            )
            business_activities = self.web_element.areatextElement(
                "#\\39 164 > textarea",
                self.app.general.business_intro,
                label="Business activities",
            )
            safety_concerns = self.web_element.areatextElement(
                "#\\39 165 > textarea", self.app.lmi.safety_concerns
            )
            # select the work location, if location equals to business location, select "primary business address' else select by line 1 of address
            is_primary = location == business_location
            if is_primary:
                sub_work_location = self.web_element.selectElement(
                    "#\\39 162 > select",
                    "Primary business address",
                    select_by_text=True,
                )
            else:
                sub_work_location = self.web_element.selectElement(
                    "#\\39 162 > select", location.line1, select_by_text=True
                )
            is_primary_check = self.web_element.checkboxElement(
                "#\\39 171 > input", is_primary, "Is this the primary work location?"
            )
            save = self.web_element.buttonElement("#saveWorkloctn", label="Save")
            wait1s = self.web_element.waitElement(1000)

            locations_actions += [
                add,
                wait,
                operating_name,
                business_activities,
                safety_concerns,
                sub_work_location,
                is_primary_check,
                save,
                wait1s,
            ]

        return locations_actions



from webform.webcommon import WebPages


class ForeignNational(WebPages):
    @property
    def actions(self):
        return [self.page1, self.page2]

    @property
    def page1(self):
        # Now we only consider one FN, so no for batch processing
        radio_id = "#\\39 181 > div > input:nth-child(4)"
        page_actions = [
            self.web_element.radioElement(radio_id, "Add FN info at one time")
        ]
        return self.web_element.pageElement(
            "#next", "#\\39 183 > input", page_actions, label="Add FN in one time"
        )

    @property
    def page2(self):
        page_actions = [
            self.web_element.buttonElement(
                "#addWorkers", label="Add a foreign national"
            ),
            self.web_element.inputElement(
                "#\\39 183 > input", self.app.personal.first_name, label="First name"
            ),
            self.web_element.inputElement(
                "#\\39 184 > input",
                self.app.personal.last_name,
                label="Last name",
                set_value=True,
            ),
            self.web_element.inputElement(
                "#\\39 185 > input", self.app.personal.dob, label="DOB", set_value=True
            ),
            self.web_element.selectElement(
                "#\\39 186 > select",
                self.app.personal.citizen,
                label="Country",
                select_by_text=True,
            ),
            self.web_element.buttonElement("#saveWorkers", label="Save"),
            self.web_element.waitForElement(
                "#addWorkers"
            ),  # wait for saving and new add appeas
            self.web_element.waitElement(
                1500
            ),  # TODO: if there are any other solutions?
        ]
        match self.app.lmiacase.stream_of_lmia:
            case "EE":
                next_page_tag = "#\\39 340 > div > input:nth-child(1)"
            case "LWS":
                next_page_tag = (
                    "#\\39 378 > div > input:nth-child(4)"  # provide accomadation No id
                )
            case "HWS":
                next_page_tag = "#\\39 349 > div > input:nth-child(4)"
            case _:
                next_page_tag = "#\\39 340 > div > input:nth-child(1)"

        return self.web_element.pageElement(
            "#next",
            next_page_tag,
            page_actions,
            label="Foreign national information",
        )



import os, sys
from pathlib import Path

HOMEDIR = Path(__file__).parents[3]
BASEDIR = Path(__file__).parents[2]
# All data directory
DATADIR = BASEDIR / "data"
# Insert the BASEDIR to system path
sys.path.insert(0, os.fspath(BASEDIR))



# import context
from pathlib import Path
import typer, json
from utils.utils import append_ext, remove_ext
from ..models import get_models
from rich.console import Console
from rich.text import Text
from rich.table import Table
from tabulate import tabulate
from termcolor import colored
from model.common.wordmaker import WordMaker
from .config import Config
from utils.utils import makeTable

# Get project's home directory,
BASEDIR = Path(__file__).parents[1]
# All data directory
DATADIR = BASEDIR / "data"

app = typer.Typer()
console = Console()


def show_exception(e: Exception):
    console.print(e, style="red")


def show_error(e):
    console.print(e, style="red")


def show_warning(msg: str):
    console.print(msg, style="yellow")


def getModel(model_name: str):
    # get model, and its instance
    models = get_models(rcic_company_id_name="", temp_num=1)
    model = models.get(model_name)
    if not model:
        show_error(f"{model_name} is not a valid model name.")
        exit(1)
    class_list = model["class_list"]
    the_class = __import__(model["path"], fromlist=class_list)
    the_model = getattr(the_class, class_list[1]) # No 2 is the Excel model
    return the_model


@app.command()
def man(model: str = typer.Argument(None, help="A model name, such as 5708")):
    """Manual of imm app"""
    models = get_models(rcic_company_id_name="", temp_num=1)
    # give specific manu for a model
    if model:
        if model not in models:
            show_error(
                f"{model} is not valid in defined models. Please use 'imm man' to check all models"
            )
            return
        help = models.get(model).get("help")
        if help:
            # print(f"{help.get('description')}")
            # print(tabulate(help.get("helps"), tablefmt="fancy_grid"))
            table = makeTable(f"Help for model {model}", help.get("helps"))
            console.print(table)

        else:
            print("There is no help info provided yet.")
    else:

        table = Table(title="Models List")
        table.add_column("Model", justify="left", style="cyan", no_wrap=True)
        table.add_column("Word", style="magenta")
        table.add_column("Pdf Form", style="magenta")
        table.add_column("Web Form ", style="magenta")
        table.add_column("Description", justify="left", style="green")

        contents = []
        for model, value in models.items():
            word_template = value.get("docx_template")
            pdf_function = value.get("pdf_function")
            web_function = value.get("web_function")
            if word_template:
                temp_name = ", ".join(word_template)
            else:
                temp_name = ""
            contents.append(
                [
                    model,
                    Path(temp_name).name,
                    pdf_function,
                    web_function,
                    value["remark"],
                ]
            )
        print(
            "Every model can do two things. 1. -m make excel document 2. -c check excel content based on the model\nFor specific help, enter mmc model_name to get the model's details.",
        )
        for content in contents:
            table.add_row(*content)

        console.print(table)


@app.command()
def make(model: str, output_excel_name: str):
    """Make excel based on model"""
    output_excel_name = append_ext(output_excel_name, ".xlsx")
    file_exists = Path(output_excel_name).exists()
    if file_exists:
        overwrite = typer.confirm(f"{output_excel_name} is existed, overwrite?")
        if not overwrite:
            return
    # create excel
    the_model = getModel(model)
    the_model(output_excel_file=output_excel_name)
    print(colored(f"{output_excel_name} has been created", "green"))


@app.command()
def check(model: str, excel_name: str, print_json: bool = False):
    """Check input excel based on model"""
    excel_name = append_ext(excel_name, ".xlsx")
    the_model = getModel(model)
    model_obj = the_model(excels=[excel_name])
    if model_obj.dict() != {}:
        print(
            json.dumps(model_obj.dict(), indent=3, default=str)
        ) if print_json else print(
            colored(f"The model has been checked and everything seems good...", "green")
        )


@app.command()
def run(model: str, excel_name: str):
    """Run app only"""
    excel_name = append_ext(excel_name, ".xlsx")
    the_model = getModel(model)
    model_obj = the_model(excels=[excel_name])
    if hasattr(model_obj, "result"):
        for k, v in model_obj.result.items():
            print(f"{k}: {v}")
    else:
        print(f"The model {model} has no excution function with return result.")


@app.command()
def word(
    model: str,
    excel_name: str,
    word_name: str = typer.Argument(
        None,
        help="Output word name. If none, willl use excel's name without ext, plus .docx",
    ),
    doctype: str = typer.Argument(
        None,
        help="Which document to generate, such as sl, ert,eet,... Use 'imm man' to check word column for details",
    ),
    rciccompany: str = typer.Option(
        "noah", help="Rcic company short name defined in docx templates"
    ),
    tempnum: int = typer.Option(1, help="template number"),
):
    """Make word based on model and input excel"""
    # check
    excel_name = append_ext(excel_name, ".xlsx")
    if word_name:
        word_name = append_ext(word_name, ".docx")
    else:
        word_name = remove_ext(excel_name) + ".docx"

    word_file_exists = Path(word_name).exists()
    excel_file_exists = Path(excel_name).exists()

    if not excel_file_exists:
        print(colored(f"{excel_name} is not existed."))
        return

    if word_file_exists:
        overwrite = typer.confirm(f"{word_name} is existed, overwrite?")
        if not overwrite:
            return

    models = get_models(rcic_company_id_name=rciccompany, temp_num=tempnum)
    template_docx_dict = models.get(model).get("docx_template")
    if not template_docx_dict:
        print(
            colored(
                f"There is no template existing in model {model.get('class_list')[0]}",
                "red",
            )
        )
        return

    # if there  is no args.document, which means the model has only one  default  dict element
    def getOnlyOneTemplate():
        the_only_one_temp = list(template_docx_dict.keys())
        temp_number = len(the_only_one_temp)
        if temp_number == 1:
            return template_docx_dict.get(the_only_one_temp[0])
        elif temp_number > 1:
            raise ValueError(
                "Word templates more than one in the model, but you did not input the doc type abbreviation"
            )
        else:
            return None

    # get model class
    try:
        # if with doc type , get the specific template, or  without args.document, get the only one
        template_docx = (
            template_docx_dict.get(doctype) if doctype else getOnlyOneTemplate()
        )
        if not template_docx:
            raise ValueError(
                f"{doctype} is not a valid document type, please check the manual by typing:'imm man'to see valid word template for each model"
            )
        elif not Path(template_docx).exists():
            raise FileNotFoundError(
                f"{template_docx} is not existed. Please check your template base"
            )

        the_model = getModel(model)
        model_instance = the_model(excels=[excel_name])
        context = model_instance.context(doc_type=doctype)
        wm = WordMaker(template_docx, context, word_name)
        wm.make()
    except Exception as e:
        show_exception(e)


@app.command()
def webform(
    model: str,
    excel_name: str,
    outputjson: str = typer.Argument(
        None,
        help="Output json file name. Without input, excel's name will be used  by adding '.json'",
    ),
    rcic: str = typer.Option(Config.rcic, help="RCIC's short name"),
    initial: bool = typer.Option(
        True, help="Is this for starting a new case. Only used for BCPNP webform."
    ),
    previous: bool = typer.Option(
        False, help="Does this client have previous BCPNP application?"
    ),
    uploaddir: str = typer.Option(
        ".", help="A directory in which all files will be uploaded."
    ),
):
    """Make json file based on model and input excel for webform filling"""
    excel_name = append_ext(excel_name, ".xlsx")
    if outputjson:
        outputjson = append_ext(outputjson, ".json")
    else:
        outputjson = remove_ext(excel_name) + ".json"
    if not uploaddir:
        show_warning(
            "You did not input uploading directory. Without it , the files in current folder will be uploaded."
        )

    key_args = {
        "output_json": outputjson,
        "upload_dir": uploaddir,
        "rcic": rcic,
        "initial": initial,
        "previous": previous,
    }
    the_model = getModel(model)
    app = the_model(excels=[excel_name])
    app.make_web_form(**key_args)


@app.command()
def pdfform(
    model: str,
    excel_name: str,
    outputjson: str = typer.Argument(
        None,
        help="Output json file name. Without input, excel's name will be used  by adding '.json'",
    ),
    rcic: str = typer.Option(Config.rcic, help="RCIC's short name"),
):
    """Make json file based on model and input excel for pdf form filling"""

    excel_name = append_ext(excel_name, ".xlsx")

    if outputjson:
        outputjson = append_ext(outputjson, ".json")
    else:
        outputjson = remove_ext(excel_name) + ".json"

    key_args = {"output_json": outputjson, "rcic_id_name": rcic}
    the_model = getModel(model)
    app = the_model(excels=[excel_name])
    app.make_pdf_form(**key_args)


if __name__ == "__main__":
    app()



import time
from abc import ABC, abstractmethod
from datetime import date, timedelta
from pywinauto.keyboard import send_keys


class Control(ABC):
    """Pdf form input control base class"""

    verbose: bool = False

    @abstractmethod
    def fill(self):
        """Fill this control"""


class Skip(Control):
    """Skip disabled control"""

    addtional_skip = 0

    def __init__(self, times=1, pause=0, verbose=False):
        self.times = times
        self.pause = pause
        self.verbose = verbose

    def fill(self):
        if self.verbose:
            print("Skip to next")
        for _ in range(self.times):
            send_keys("{TAB}", pause=self.pause)
        Skip.addtional_skip += self.times


class Button(Control):
    def __init__(self, pause=0, verbose=False) -> None:
        self.pause = pause
        self.verbose = verbose

    def fill(self):
        if self.verbose:
            print("Click button")
        send_keys("{ENTER}", pause=self.pause)


class Pause(Control):
    """Pause for a moment"""

    def __init__(self, seconds, verbose=False):
        self.seconds = seconds
        self.verbose = verbose

    def fill(self):
        if self.verbose:
            print(f"Pause {self.seconds} seconds")
        time.sleep(self.seconds)


class TextField(Control):
    """Text field"""

    def __init__(self, data, pause=0, verbose=False):
        """data: text to fill"""
        self.data = data
        self.pause = pause
        self.verbose = verbose

    def fill(self):
        if self.verbose:
            print(f"Fill text field with {self.data}")
        if type(self.data) == list:
            for d in self.data:
                time.sleep(3)
                send_keys(str(d).replace(" ", "{SPACE}"))
            send_keys("{TAB}")
        else:
            send_keys(str(self.data).replace(" ", "{SPACE}"), pause=self.pause)
            send_keys("{TAB}")


class RadioButton(Control):
    """Radio button"""

    def __init__(self, data, pause=0, verbose=False):
        """data: False for No, True for yes"""
        self.data = data
        self.pause = pause
        self.verbose = verbose

    def fill(self):
        if self.data:
            key = "{VK_RIGHT}"
        else:
            key = "{VK_SPACE}"
        if self.verbose:
            print(f"Fill radio button with {self.data}, press {key} key")
        send_keys(key, pause=self.pause)
        send_keys("{TAB}")


# Check one button in a button list. Check the position with True
class RadioButtonList(Control):
    """Radio button List"""

    def __init__(self, pause=0, verbose=False, position=0):
        self.pause = pause
        self.verbose = verbose
        self.position = position

    def fill(self):
        if self.position > 0:
            for _ in range(1, self.position + 1):
                send_keys("{VK_RIGHT}", pause=self.pause)
        else:
            send_keys("{VK_SPACE}", pause=self.pause)

        if self.verbose:
            print(f"Fill radio button with checked in position {self.position}")

        send_keys("{TAB}")


class CheckBox(Control):
    """Check box"""

    def __init__(self, data, pause=0, verbose=False):
        self.data = data
        self.pause = pause
        self.verbose = verbose

    def fill(self):
        if self.verbose:
            print(f"Fill check box with {self.data}")
        if self.data:
            send_keys("{VK_SPACE}")
            send_keys("{TAB}")
        else:
            send_keys("{TAB}")
        time.sleep(self.pause)


class OutputInfo(Control):
    """Output log info"""

    def __init__(self, info, verbose=False):
        self.info = info
        self.verbose = verbose

    def fill(self):
        if self.verbose:
            print(f"------- {self.info} -------")


class DropdownList(Control):
    """Dropdown list"""

    def __init__(self, data, key, num, pause=0.01, verbose=False):
        self.data = data
        self.key = key
        self.num = num
        self.pause = pause  # pause seconds after press
        self.verbose = verbose

    def fill(self):
        # key, num = self.key_presses()
        if self.verbose:
            print(
                f"Select dropdown list with {self.data}, press {self.key} {self.num} time(s)"
            )
        for _ in range(self.num):
            send_keys(self.key, pause=self.pause)

        send_keys("{TAB}")


class DateField(Control):
    """Date field"""

    def __init__(self, the_date, noday: bool = False, pause=0.2, verbose=False):
        self.date = the_date or date.today() + timedelta(days=1)
        self.noday = noday
        self.pause = pause
        self.verbose = verbose

    def fill(self):
        self.date = (
            self.date.strftime("%Y-%m-%d") if type(self.date) == date else self.date
        )
        year, month, day = self.date.split("-")
        if self.noday:
            if self.verbose:
                print(f"Fill date {year}-{month}")
            send_keys(year + month, pause=self.pause)

        else:
            if self.verbose:
                print(f"Fill date {year}-{month}-{day}")
            send_keys(year, pause=self.pause)
            send_keys(month, pause=self.pause)
            send_keys(day, pause=self.pause)
        send_keys("{TAB}")



from webform.webcommon import WebPages, Page

# 1. Accomadation for LWS
class Accommodation(WebPages):
    @property
    def actions(self):
        return [self.page1, *self.page2]

    @property
    def page1(self):
        provide_accommodation_id = (
            "#\\39 378 > div > input:nth-child(1)"
            if self.app.emp5627.provide_accommodation
            else "#\\39 378 > div > input:nth-child(4)"
        )
        provide_accommodation = self.web_element.radioElement(
            provide_accommodation_id,
            label="Yes" if self.app.emp5627.provide_accommodation else "No",
        )

        # next page tag: if provide, wait for rent amount id, if no then wait for the id of explaination
        next_page_tag = (
            "#\\39 382 > input"
            if self.app.emp5627.provide_accommodation
            else "#\\39 379 > textarea"
        )
        return Page(
            [provide_accommodation],
            "#next",
            next_page_tag,
            label="Provide TFW accommodation?",
        ).page

    @property
    def page2(self):
        """includes sub pages if yes to provide the accommodation"""
        # question for if exempted from CAP
        is_exempted_id = (
            "#\\39 391 > div > input:nth-child(1)"
            if self.app.emp5627.cap_exempted
            else "#\\39 391 > div > input:nth-child(4)"
        )
        is_exempted = self.web_element.radioElement(
            is_exempted_id, "Yes" if self.app.emp5627.cap_exempted else "No"
        )

        if self.app.emp5627.provide_accommodation:
            sub_p1 = self.accommodation_details1()
            sub_p2 = self.accommodation_details2(is_exempted)
            return [sub_p1, sub_p2]
        else:
            explain_why_not_provide = self.web_element.areatextElement(
                "#\\39 379 > textarea",
                self.app.emp5627.description,
                label="Explain how to assist TFW for accommodation",
            )
            # if exempted from CAP, then goto hours and pay, no id,
            # if not, then goto ask if it is seasonal, no id
            next_page_tag = (
                "#\\39 433 > div > input:nth-child(4)"
                if self.app.emp5627.cap_exempted
                else "#\\39 395 > div > input:nth-child(4)"
            )
            return [
                Page(
                    [explain_why_not_provide, is_exempted],
                    "#next",
                    next_page_tag,
                    label="Accommodation explaination",
                ).page
            ]

    def accommodation_details1(self):
        accommodation_types = {
            "apartment": "#\\39 384 > div > input:nth-child(1)",
            "dorm": "#\\39 384 > div > input:nth-child(4)",
            "house": "#\\39 384 > div > input:nth-child(7)",
            "other": "#\\39 384 > div > input:nth-child(7)",
        }
        a_type = self.app.emp5627.accommodation_type
        accommodation_type = self.web_element.radioElement(
            accommodation_types.get(a_type), label=a_type
        )

        amount = self.web_element.inputElement(
            "#\\39 382 > input", str(self.app.emp5627.rent_amount), label="rent amount"
        )
        unit = self.web_element.selectElement(
            "#\\39 383 > select", self.app.emp5627.rent_unit, select_by_text=True
        )
        next_page_tag = (
            "#\\39 391 > div > input:nth-child(4)"  # exempted from CAP no id
        )
        return Page(
            [accommodation_type, amount, unit],
            "#next",
            next_page_tag,
            label="Accommodation details type and rent amount",
        ).page

    def accommodation_details2(self, is_exempted):
        accommodation_actions = [
            self.web_element.inputElement(
                "#\\39 386 > input",
                self.app.emp5627.bedrooms,
                label="How many bed rooms",
            ),
            self.web_element.inputElement(
                "#\\39 387 > input",
                self.app.emp5627.people,
                label="How many occupants?",
            ),
            self.web_element.inputElement(
                "#\\39 388 > input",
                self.app.emp5627.bathrooms,
                label="How many bathrooms?",
            ),
            self.web_element.areatextElement(
                "#\\39 389 > textarea",
                self.app.emp5627.other or "N/A",
                label="other description",
            ),
            is_exempted,
        ]
        #  if cap exempted, go directly to hours and pay. no id
        # if not, goto ask if in seasonal industry. No id
        next_page_tag = (
            "#\\39 433 > div > input:nth-child(4)"
            if self.app.emp5627.cap_exempted
            else "#\\39 395 > div > input:nth-child(4)"
        )
        return Page(
            accommodation_actions,
            "#next",
            next_page_tag,
            label="Accommodation details 2",
        ).page



# import context
from pprint import pprint
import typer
from rich.console import Console
from rich.text import Text
from getpass import getpass
from apps.api.login import login, get_session
import requests
from typing import List
from apps.api.hashing import Hash

app = typer.Typer()
console = Console()

headers = {"accept": "application/json", "Content-Type": "application/json"}


def get_id_by_email(session: requests.Session, email: str):
    users = session.get("http://localhost:8000/user/")
    for user in users.json()["data"]:
        if email == user.get("email"):
            return user.get("_id")


def delete_by_email(session: requests.Session, email: str):
    user_id = get_id_by_email(session, email)
    res = session.delete(f"http://localhost:8000/user/{user_id}")
    result = res.json()
    if result.get("status_code") == 204:
        f"Record with id {user_id} has been deleted.",
        return user_id
    elif result.get("status_code") == 401:
        print(res.json().get("detail"))
    else:
        print(f"No record with email as {email}")


def delete_by_id(session: requests.Session, id: str):
    res = session.delete(f"http://localhost:8000/user/{id}")
    status = res.json()
    if status["status_code"] == 204:
        print(
            f"Record with id {id} has been deleted.",
        )
    elif status["status_code"] == 401:
        print("Error:", status["detail"])


@app.command()
def register(
    username: str = typer.Argument(None, help="User name"),
    email: str = typer.Argument(None, help="Email, used as login name"),
    phone: str = typer.Argument(None, help="Telephone"),
):
    psw1 = getpass()
    psw2 = getpass("Input password again: ")
    if psw1 != psw2:
        raise ValueError("The two passwords are not same.")
    user = {
        "username": username,
        "email": email,
        "phone": phone,
        "password": psw2,
    }
    r = requests.post("http://localhost:8000/user/", json=user)
    if r.status_code == 200:
        print(r.json())
    else:
        print(r.status_code, r.json["detail"])


@app.command()
def show(
    email: str = typer.Argument(None, help="Email, used as login name"),
    id: str = typer.Option(None, help="id, record id"),
):
    session = get_session()
    if id:
        r = session.get(f"http://localhost:8000/user/{id}")
    elif email:
        id = get_id_by_email(session, email)
        if id:
            r = session.get(f"http://localhost:8000/user/{id}")
        else:
            print(f"No user with email as {email}")
            exit(1)
    else:
        r = session.get("http://localhost:8000/user/")
    if r.status_code == 200:
        pprint(r.json()["data"])
    else:
        print(r.status_code, r.json().get("detail"))


@app.command()
def delete(
    email: str = typer.Argument(None, help="Email, used as login name"),
    id: str = typer.Option(None, help="id, record id"),
):
    session = get_session()
    if email:
        delete_by_email(session, email)
    elif id:
        delete_by_id(session, id)
    else:
        print("You have to either assign email or _id")


@app.command()
def role(user_email: str, role: str):
    session = get_session()
    data = {"user_email": user_email, "role": role}
    r = session.post("http://localhost:8000/user/role", json=data)
    print(r.status_code, r.json())


@app.command()
def permission(
    user_email: str,
    remove: bool = typer.Option(False),
    permission: List[str] = typer.Option(None),
):
    session = get_session()
    data = {"user_email": user_email, "permissions": list(permission), "remove": remove}
    r = session.post("http://localhost:8000/user/permission", json=data)
    print(r.status_code, r.json())


@app.command()
def password(user_email: str, password: str):
    session = get_session()
    _id = get_id_by_email(session, user_email)
    query = {"password": password}
    r = session.put(f"http://localhost:8000/user/password/{_id}", params=query)
    print(
        "Your password has been updated successfully. Don't forget to update the imm_password variable  in .immenv file  "
    )


if __name__ == "__main__":
    try:
        app()
    except Exception as e:
        print(e)



from .context import DATADIR
from typing import List
from model.common.commonmodel import CommonModel, BuilderModel
from model.recruit.jobofferdata import JobOffer, General, ErAddress, ErAddresses
from model.common.address import Address, Addresses
from model.common.person import Person
from model.common.wordmaker import WordMaker
import os
from pydantic import BaseModel


class Personal(Person):
    def __str__(self):
        return self.full_name


class JobofferModel(BaseModel, BuilderModel):
    general: General
    joboffer: JobOffer
    eraddress: List[ErAddress]
    personal: Personal
    address: List[Address]

    def context(self, *args, **kwargs):
        context = {
            **self.dict(),
            "offer_date": self.joboffer.offer_date.strftime("%b %d, %Y"),
            "term": self.joboffer.term,
            "has_benefits": self.joboffer.has_benefits,
            "benefits": self.joboffer.benefits.lower(),
            "full_part_time": self.joboffer.full_part_time,
            "workingaddress": ErAddresses(self.eraddress).working,
            "address_line1": Addresses(self.address).residential.line1,
            "address_line2": Addresses(self.address).residential.line2,
            "work_start_date": self.joboffer.work_start_date.strftime("%b %d, %Y")
            if self.joboffer.work_start_date
            else None,
            "vacation_pay_percentage": self.joboffer.vacation_pay_percent,
        }
        return context

    def make_pdf_form(self, *args, **kwargs):
        pass

    def make_web_form(self, *args, **kwargs):
        pass


class JobofferModelE(CommonModel, JobofferModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/pa.xlsx", "excel/er.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())


class JobofferModelDocxAdapater:
    """This is an adapater to bridging job ad model data and docx data"""

    def __init__(self, joboffer_obj: JobofferModel):
        # get original obj, which will be used to generate some value based on it's object methods.
        # 此处用来处理list里面的一些内容。
        self.joboffer_obj = joboffer_obj
        eraddresses = ErAddresses(self.joboffer_obj.eraddress)
        self.joboffer_obj.eraddress = eraddresses.working

        addresses = Addresses(self.joboffer_obj.address)
        self.joboffer_obj.address = addresses.residential

    def make(self, output_docx, template_no=None):
        file_name = "word/joboffer" + str(template_no) + ".docx"
        template_path = os.path.abspath(os.path.join(DATADIR, file_name))
        wm = WordMaker(template_path, self.joboffer_obj, output_docx)
        wm.make()



import context
from datetime import date
import httpx
import json
from source.excel import Excel
from pprint import pprint


def getJsonData(excel_file_name):
    e = Excel(excel_name=excel_file_name)
    return json.loads(e.json)


# get user
def login(email, password):
    client = httpx.Client()
    headers = {
        "accept": "application/json",
        "Content-Type": "application/x-www-form-urlencoded",
    }
    r = client.post(
        "http://localhost:8000/login/",
        json={"username": email, "password": password},
        # headers=headers,
    )
    print(r.content)
    token = json.loads(r.content)["access_token"]
    client.headers.update({"authorization": token})
    print(r.content)
    return client


client = login("jackyson@gmail7.com", "777")
r = client.get("http://localhost:8000//user")
print(r)

# # read
# r = httpx.get("http://localhost:8000/user/630aa43fb5e1ee46a63716f6")
# print(r.status_code, r.content)

# # create user
# jacky = {
#     "username": "may",
#     "email": "jacky2@gmail.com",
#     "phone": "7783215110",
#     "password": "222",
# }
# r = httpx.post("http://localhost:8000/user/", json=jacky)
# print(r.status_code, r.content)

# update
# jacky = {
#     "username": "lele",
#     "email": "jackyzhang1969@gmail.com",
#     "phone": "7783215110",
#     "password": "xxxxx",
# }
# r = httpx.put("http://localhost:8000/user/630aa056f0e34b68a8d7b9bf", json=jacky)
# print(r.status_code, r.content)

# # delete
# r = httpx.delete("http://localhost:8000/user/630aa056f0e34b68a8d7b9bf")
# print(r.status_code, r.content)



from typing import List, Optional
from model.common.address import Address
from model.common.educationbase import EducationBase
from model.common.phone import Phone
from model.common.commonmodel import CommonModel, BuilderModel
from model.common.employmentbase import EmploymentBase
from model.common.jobofferbase import JobofferBase
from model.common.id import ID
from model.common.person import Person
from model.common.language import LanguageBase
from datetime import date, datetime
from pydantic import BaseModel, EmailStr, root_validator, validator
from model.common.utils import normalize
from termcolor import colored
from webform.bcpnp.joboffer_reg import JobofferReg
from webform.bcpnp.login import Login
from webform.bcpnp.registrant import Registrant
from webform.bcpnp.education_reg import EducationReg
from webform.bcpnp.employment import EmploymentReg
from webform.bcpnp.language import LanguageReg
from webform.bcpnp.joboffer_reg import JobofferReg
from webform.bcpnp.submit import Submit
from webform.bcpnp.register import Register
import json


class PersonId(ID):
    pass


class Personal(Person):
    email: EmailStr
    used_last_name: Optional[str]
    used_first_name: Optional[str]
    place_of_birth: str
    country_of_birth: str
    did_eca: bool
    eca_supplier: Optional[str]
    eca_number: Optional[str]
    ita_assessed: bool
    ita_assess_number: Optional[str]

    _normalize_used_first_name = validator(
        "used_first_name", allow_reuse=True, check_fields=False
    )(normalize)
    _normalize_used_last_name = validator(
        "used_last_name", allow_reuse=True, check_fields=False
    )(normalize)

    @root_validator
    def checkECA(cls, values):
        did_eca = values.get("did_eca")
        eca_supplier = values.get("eca_supplier")
        eca_number = values.get("eca_number")

        if did_eca and not all([eca_supplier, eca_number]):
            raise ValueError(f"Did ECA,but not input ECA number and/or ECA supplier")
        return values

    @root_validator
    def checkITA(cls, values):
        ita_assessed = values.get("ita_assessed")
        ita_assess_number = values.get("ita_assess_number")

        if ita_assessed and not ita_assess_number:
            raise ValueError(
                f"You selected ITA as Yes,but not input ITA assessment number"
            )
        return values

    @property
    def user_id(self):
        dob = self.dob.strftime(("%Y-%m-%d"))
        return (
            self.last_name[0].upper()
            + self.first_name[0]
            + dob.split("-")[0]
            + dob.split("-")[1]
            + dob.split("-")[2]
        )

    @property
    def password(self):
        return "Super" + str(datetime.today().year) + "!"


class BcpnpModelPro(BaseModel, BuilderModel):
    personal: Personal
    address: List[Address]
    phone: List[Phone]
    personid: List[PersonId]

    def context(self, *args, **kwargs):
        pass

    def make_pdf_form(self, *args, **kwargs):
        pass

    def make_web_form(
        self, output_json="", initial=True, previous=False, *args, **kwargs
    ):
        actions = Register(self).fill()
        with open(output_json, "w") as f:
            json.dump(actions, f, default=str, indent=3)
        print(colored(f"{output_json} has been created.", "green"))


class BcpnpModelProE(CommonModel, BcpnpModelPro):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/pa.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from pydantic import BaseModel, EmailStr
from model.common.commonmodel import CommonModel
import os
from typing import List
from pydantic import BaseModel
from model.common.advertisement import (
    Advertisement,
    InterviewRecord,
    RecruitmentSummary,
)


class LmiaApplication(BaseModel):
    advertisement: List[Advertisement]
    interviewrecord: List[InterviewRecord]
    recruitmentsummary: RecruitmentSummary


class LmiaApplicationE(CommonModel):
    advertisement: List[Advertisement]
    interviewrecord: List[InterviewRecord]
    recruitmentsummary: RecruitmentSummary

    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/recruitment.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from pdfform.tr.common import TrCommon
from pdfform.tr import option_lists


class FormBuilder5708(TrCommon):
    """Form builder"""
    def start(self):
        """Add initial 3 skips"""
        self.form.add_skip(3)
        
    def add_header(self):
        """Add header section items"""
        personal = self.applicant.personal
        self.form.add_text(personal.uci)
        self.form.add_dropdown(
            self.applicant.trcasein.service_in, option_lists.service_language
        )
        if (
            self.applicant.vrincanada.application_purpose
            == "apply or extend visitor record"
        ):
            self.form.add_checkbox(True)
        else:
            self.form.add_skip(1)
        if self.applicant.vrincanada.application_purpose == "restore status as visotor":
            self.form.add_checkbox(True)
        else:
            self.form.add_skip(1)
        if self.applicant.vrincanada.application_purpose == "TRP":
            self.form.add_checkbox(True)
        else:
            self.form.add_skip(1)

    def add_visit_detail(self):
        """Add visit details"""
        self.form.add_info("details of visit section")

        visit = self.applicant.vrincanada
        self.form.add_dropdown(visit.visit_purpose, option_lists.purpose_of_vist)
        if visit.visit_purpose.lower() == "other":
            self.form.add_text(visit.other_explain)
        else:
            self.form.add_skip(1)
        self.form.add_date(visit.start_date)
        self.form.add_date(visit.end_date)
        self.form.add_text(str(visit.funds_available))
        self.form.add_dropdown(visit.paid_person, option_lists.expense_by)
        if visit.paid_person.lower() == "other":
            self.form.add_text(visit.other_payer_explain)
        else:
            self.form.add_skip(1)

        self.form.add_text(visit.name1)
        self.form.add_text(visit.relationship1)
        self.form.add_text(visit.address1)
        self.form.add_text(visit.name2)
        self.form.add_text(visit.relationship2)
        self.form.add_text(visit.address2)

    def get_form(self):
        self.start()
        self.add_header()
        self.add_personal_detail()
        self.add_language()
        self.add_passport()
        self.add_national_id()
        self.add_uspr_card()
        self.add_contact_information(form_type="5708")
        self.add_coming_into_canada()
        self.add_visit_detail()
        self.add_education()
        self.add_employment()
        self.add_background()
        self.add_signature()
        return self.form



from typing import Optional
from pydantic import BaseModel, EmailStr


class User(BaseModel):
    username: str
    email: EmailStr
    phone: Optional[str]
    password: str
    role: str = "user"
    permission: list = []



from model.common.commonmodel import FormBuilderBase
from pdfform.jsonmaker import JsonMaker
from model.common.rcic import Rcics
from datetime import date


class FormBuilder5476(FormBuilderBase):
    def __init__(self, d5476: object, rcic_id_name: str, appoint: bool = True):
        self.d5476 = d5476
        self.form = JsonMaker()
        self.appoint = appoint
        self.rcic_id_name = rcic_id_name
        self.rcic = self.get_rcic(rcic_id_name)
        self.text_speed = 0.01
        self.skip_speed = 0.01

    def get_rcic(self, rcic_id_name):
        return Rcics(self.d5476.rciclist).getRcicByIdName(rcic_id_name)

    def start(self):
        self.form.add_skip(5, pause=self.skip_speed)

    def pick_rep_purpose(self):
        if self.appoint:
            self.form.add_checkbox(True)
            self.form.add_skip(1, pause=self.skip_speed)
        else:
            self.form.add_skip(1, pause=self.skip_speed)
            self.form.add_checkbox(True)

    def add_applicant(self):
        self.form.add_text(self.d5476.personal.last_name, pause=self.text_speed)
        self.form.add_text(self.d5476.personal.first_name, pause=self.text_speed)
        self.form.add_date(self.d5476.personal.dob, pause=self.text_speed)
        # currently ignore the application number
        self.form.add_skip(2, pause=self.skip_speed)
        self.form.add_text(
            self.d5476.personal.uci, pause=self.text_speed
        ) if self.d5476.personal.uci else self.form.add_skip(1, pause=self.skip_speed)

    # following adding applicant, there are add rep or cancel rep
    def add_rep(self):
        if not self.rcic:
            raise ValueError(f"RCIC with id_name {self.rcic_id_name} is not existed.")
        self.form.add_text(self.rcic.last_name, pause=self.text_speed)
        self.form.add_text(self.rcic.first_name, pause=self.text_speed)
        # skip to organization
        self.form.add_skip(4, pause=self.skip_speed)
        self.form.add_checkbox(True)
        self.form.add_text(self.rcic.rcic_number, pause=self.text_speed)
        # employer name
        self.form.add_text(self.rcic.employer_legal_name, pause=self.text_speed)
        # skip to mailing address
        self.form.add_skip(2, pause=self.skip_speed)
        # mailing address
        self.form.add_text(self.rcic.unit, pause=self.text_speed)
        self.form.add_text(self.rcic.street_number, pause=self.text_speed)
        self.form.add_text(self.rcic.street_name, pause=self.text_speed)
        self.form.add_text(self.rcic.city, pause=self.text_speed)
        self.form.add_text(self.rcic.province, pause=self.text_speed)
        self.form.add_text(self.rcic.country, pause=self.text_speed)
        self.form.add_text(self.rcic.post_code, pause=self.text_speed)
        self.form.add_text(self.rcic.country_code, pause=self.text_speed)
        self.form.add_text(self.rcic.phone, pause=self.text_speed)
        # skip to email
        self.form.add_skip(2, pause=self.skip_speed)
        self.form.add_text(self.rcic.email, pause=self.text_speed)

        self.form.add_skip(1, pause=self.skip_speed)
        self.form.add_text(date.today().strftime("%Y-%m-%d"), pause=self.text_speed)
        # self.form.add_skip(1, pause=self.skip_speed)

        # skip to applicant date
        self.form.add_skip(7, pause=self.skip_speed)
        self.form.add_text(date.today().strftime("%Y-%m-%d"), pause=self.text_speed)

    def cancel_rep(self):
        # skip from applicant to cancel rep
        self.form.add_skip(24, pause=self.skip_speed)
        self.form.add_text(self.rcic.last_name, pause=self.text_speed)
        self.form.add_text(self.rcic.first_name, pause=self.text_speed)
        self.form.add_text(self.rcic.employer_legal_name, pause=self.text_speed)

        # skip to applicant date
        self.form.add_skip(4, pause=self.skip_speed)
        self.form.add_text(date.today().strftime("%Y-%m-%d"), pause=self.text_speed)

    def get_form(self):
        self.start()
        self.pick_rep_purpose()
        self.add_applicant()
        if self.appoint:
            self.add_rep()
        else:
            self.cancel_rep()
        return self.form



from utils.utils import best_match
from typing import List, Union


class JsonMaker:
    """According to data model, generate json file for pdf form filling"""

    def __init__(self):
        self.actions = []

    def _getKey_Num(self, data: str, options: List[str], like=False):
        s_char = data[0].lower()
        count = 0
        matched_one = best_match(data, options)
        for elem in options:
            if elem[0].lower() == s_char:
                count += 1
                if elem.lower() == data.lower():
                    break
                elif like and f"({data})" in elem:
                    break
                elif elem.lower() == matched_one.lower():
                    break
        return (s_char, count)

    def add_skip(self, times: int, pause: float = 0):
        self.actions.append({"action_type": "Skip", "times": times, "pause": pause})

    def add_button(self, pause: float = 0):
        self.actions.append({"action_type": "Button", "pause": pause})

    def add_text(self, data: Union[str, list], pause: float = 0):
        data = data or ""
        self.actions.append({"action_type": "TextField", "data": data, "pause": pause})

    def add_radio(self, data: bool, pause: float = 0):
        self.actions.append(
            {
                "action_type": "RadioButton",
                "data": data,
                "pause": pause,
            }
        )

    def add_radio_list(self, pause: float = 0, position=0):
        self.actions.append(
            {
                "action_type": "RadioButtonList",
                "pause": pause,
                "position": position,
            }
        )

    def add_checkbox(self, data: bool, pause: float = 0):
        self.actions.append({"action_type": "CheckBox", "data": data, "pause": pause})

    def add_info(self, info: str, pause: float = 0):
        self.actions.append({"action_type": "OutputInfo", "info": info, "pause": pause})

    def add_dropdown(self, data: str, options: list[str], like=False, pause: float = 0):
        key, num = self._getKey_Num(data, options, like=like)
        self.actions.append(
            {
                "action_type": "DropdownList",
                "data": data,
                "key": key,
                "num": num,
                "like": like,
                "pause": pause,
            }
        )

    def add_date(self, the_date, noday: bool = False, pause: float = 0):
        self.actions.append(
            {
                "action_type": "DateField",
                "date": the_date,
                "noday": noday,
                "pause": pause,
            }
        )

    def add_pause(self, pause: float = 1):
        self.actions.append({"action_type": "Pause", "pause": pause})



import requests
from pprint import pprint
from getpass import getpass
from apps.config import config, path as IMM_ENV_FILE


def login(email, password):
    session = requests.Session()
    payload = {"username": email, "password": password}
    res = session.post("http://127.0.0.1:8000/login", data=payload)
    if res.status_code == 404:
        print(res.json().get("detail"))
        exit(1)
    session.headers.update(
        {
            "Content-Type": "application/json",
            "Authorization": f"{res.json()['token_type']} {res.json()['access_token']}",
        }
    )
    return session


def get_session():
    if config.get("imm_account") == None or config.get("imm_password") == None:
        imm_account = input("Input your email: ")
        imm_password = getpass()
        try:
            session = login(imm_account, imm_password)
            with open(IMM_ENV_FILE, mode="a") as f:
                f.write(f"\nimm_account={imm_account}\nimm_password={imm_password}")
            return session
        except Exception as e:
            print(e.args)
            exit(1)
    else:
        imm_account = config.get("imm_account")
        imm_password = config.get("imm_password")
        return login(imm_account, imm_password)



from pydantic import BaseModel
from typing import Optional

class MediaAccount(BaseModel):
    media:Optional[str]
    account:Optional[str]
    password:Optional[str]
    security_answer:Optional[str]
    remark:Optional[str]
    


from pydantic import BaseModel, validator, EmailStr, root_validator
from model.common.utils import makeList
from model.common.person import Person
from model.common.employmentbase import EmploymentBase
from model.common.utils import checkRow
from pydantic import BaseModel, root_validator, validator, EmailStr
from typing import Optional
from model.common.utils import makeList, checkRow
from model.common.person import Person
from model.common.employmentbase import EmploymentBase
from model.common.educationbase import EducationBase


class PersonalAssess(BaseModel):
    self_description: Optional[str]
    skill_list: list
    activity: Optional[list]

    _str2bool_activity = validator("activity", allow_reuse=True, pre=True)(makeList)
    _str2bool_skill_list = validator("skill_list", allow_reuse=True, pre=True)(makeList)


class Personal(Person):
    email: Optional[EmailStr]

    @property
    def birth_day(self):
        return self.dob.strftime("%b %d, %Y")


class Education(EducationBase):
    city: str
    country: str
    description: Optional[str]

    @root_validator
    def checkRowCompletion(cls, values):
        all_fields = [
            "start_date",
            "end_date",
            "school_name",
            "education_level",
            "field_of_study",
            "city",
            "country",
            "description",
        ]
        required_fields = [
            "start_date",
            "school_name",
            "education_level",
            "field_of_study",
            "city",
            "country",
        ]

        checkRow(values, all_fields, required_fields)

        return values


class Language(BaseModel):
    reading: str
    writting: str
    listening: str
    speaking: str
    test_type: str
    remark: Optional[str]

    @root_validator
    def checkRowCompletion(cls, values):

        all_fields = [
            "reading",
            "writting",
            "listening",
            "speaking",
            "test_type",
            "remark",
        ]
        required_fields = ["reading", "writting", "listening", "speaking", "test_type"]
        checkRow(values, all_fields, required_fields)

        return values


class Employment(EmploymentBase):
    department: Optional[str]
    duties: list
    company_brief: str
    fullname_of_certificate_provider: Optional[str]
    position_of_certificate_provider: Optional[str]
    department_of_certificate_provider: Optional[str]
    phone_of_certificate_provider: Optional[str]
    email_of_certificate_provider: Optional[EmailStr]
    employment_certificate: bool

    _normalize_duties = validator("duties", allow_reuse=True, pre=True)(makeList)

    @root_validator
    def checkCanadaProvince(cls, values):
        country = values.get("country")
        province = values.get("province")
        if country.lower() == "canada" and province not in [
            "AB",
            "BC",
            "MB",
            "NB",
            "NL",
            "NS",
            "NT",
            "NU",
            "ON",
            "PE",
            "QC",
            "SK",
            "YT",
        ]:
            raise ValueError(
                f'Since country is Canada is, the province must be one of  "AB","BC","MB","NB","NL","NS","NT","NU","ON","PE","QC","SK","YT"'
            )
        return values

    @root_validator
    def checkCompletion(cls, values):
        all_fields = [
            "job_title",
            "noc_code",
            "weekly_hours",
            "company",
            "city",
            "province",
            "country",
            "department",
            "duties",
            "company_brief",
            "fullname_of_certificate_provider",
            "position_of_certificate_provider",
            "department_of_certificate_provider",
            "phone_of_certificate_provider",
            "email_of_certificate_provider",
            "employment_certificate",
        ]
        required_fields = [
            "job_title",
            "noc_code",
            "weekly_hours",
            "company",
            "city",
            "province",
            "country",
            "duties",
            "company_brief",
            "fullname_of_certificate_provider",
            "position_of_certificate_provider",
            "phone_of_certificate_provider",
            "email_of_certificate_provider",
            "employment_certificate",
        ]

        checkRow(values, all_fields, required_fields)

        return values



from fastapi import APIRouter, Depends, HTTPException, status
from fastapi.security import OAuth2PasswordRequestForm
from apps.config import db
from apps.api.hashing import Hash
from datetime import timedelta, datetime
from jose import JWTError, jwt
from typing import Union

user_collection = db.user

SECRET_KEY = "09d25e094faa6ca2556c818166b7a9563b93f7099f6f0f4caa6cf63b88e8d3e7"
ALGORITHM = "HS256"
ACCESS_TOKEN_EXPIRE_MINUTES = 30


def create_access_token(data: dict, expires_delta: Union[ timedelta,None] = None):
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)
    return encoded_jwt


def verify_token(token: str, credentials_exception):
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        email: str = payload.get("sub")
        if email is None:
            raise credentials_exception
        return payload
    except JWTError:
        raise credentials_exception


router = APIRouter(
    prefix="/login",
    tags=["Authentication"],
    responses={404: {"description": "Nof found"}},
)


@router.post("/")
async def login(request: OAuth2PasswordRequestForm = Depends()):
    user = user_collection.find_one({"email": request.username})
    if not user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"{request.username} is not registered in the system.",
        )
    if not Hash.verify(request.password, user["password"]):
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Password is incorrect"
        )

    access_token = create_access_token(
        data={
            "sub": user["email"],
            "role": user.get("role"),
            "permission": user.get("permission"),
        }
    )
    return {"access_token": access_token, "token_type": "bearer"}



from pymongo import MongoClient
import os
import dotenv

path = os.path.abspath(os.path.join(os.path.expanduser("~"), ".immenv"))
config = dotenv.dotenv_values(path)

# Mongodb
account = config.get("MongoDBUser")
password = config.get("MongoDBPassword")
connection = f"mongodb+srv://{account}:{password}@noah.yi5fo.mongodb.net/test?retryWrites=true&w=majority"
client = MongoClient(connection)
database = "test"
db = client[database]

# imm account
imm_account: str = config.get("imm_user")
imm_password: str = config.get("imm_password")


class Default:
    rcic = "jacky"
    rciccompany = "noah"
    temp_num = 1  # for word generation using template
    uploaddir = "."  # for webform, uploading all dir's file
    initial = True  # Only for BCPNP webform to check if it is initial reg or app
    previous = False  # Only for BCPNP webform to check if there is previous application
    user_permission = ["make", "check"]



import context
from fastapi import FastAPI

# import uvicorn
from apps.api.routers import user, case, authentication

app = FastAPI()

app.include_router(authentication.router)
app.include_router(user.router)
app.include_router(case.router)

# uvicorn.run(app, host="localhost", port=8000)



from .context import DATADIR
from typing import List
from model.common.commonmodel import CommonModel, BuilderModel
from model.lmia.data import LmiaCase, Contact, General
from model.common.advertisement import (
    Advertisement,
    InterviewRecord,
    InterviewRecords,
    Advertisements,
)
from model.common.person import Person
from model.common.contact import Contacts
from model.common.jobofferbase import JobofferBase
import os
from model.common.wordmaker import WordMaker
from pydantic import BaseModel


class Personal(Person):
    def __str__(self):
        return self.full_name


class Joboffer(JobofferBase):
    pass


class RecruitmnetSummaryModel(BaseModel, BuilderModel):
    general: General
    contact: List[Contact]
    lmiacase: LmiaCase
    advertisement: List[Advertisement]
    interviewrecord: List[InterviewRecord]
    joboffer: Joboffer

    @property
    def selected_contact(self):
        contacts = Contacts(self.contact)
        return contacts.preferredContact

    @property
    def summary(self):
        return InterviewRecords(self.interviewrecord)

    @property
    def advertisements(self):
        records = []
        i = 1
        for adv in self.advertisement:
            records.append({**{"days": adv.days}, **adv.dict()})
            i += 1
        return records

    def context(self, *args, **kwargs):
        interview = InterviewRecords(self.interviewrecord)
        primary_contact = Contacts(self.contact).primary
        return {
            **self.dict(),
            "num_of_job_posts": Advertisements(self.advertisement).amount,
            "adv_summary": [
                {
                    "start_date": a.start_date,
                    "end_date": a.end_date,
                    "media": a.media,
                    "days": a.days,
                }
                for a in self.advertisement
            ],
            "high_wage": True
            if float(self.joboffer.hourly_rate) >= self.lmiacase.provincial_median_wage
            else False,
            "rs": {
                "resume_num": interview.resume_num,
                "canadian_num": interview.canadian_num,
                "unknown_num": interview.unknown_num,
                "foreigner_num": interview.foreigner_num,
                "total_canadian": interview.total_canadian,
                "total_interviewed_canadians": interview.total_interviewed_canadians,
                "canadian_records": interview.canadian_records,
            },
            "primary_contact": primary_contact,
        }

    def make_pdf_form(self, *args, **kwargs):
        pass

    def make_web_form(self, *args, **kwargs):
        pass


class RecruitmnetSummaryModelE(CommonModel, RecruitmnetSummaryModel):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/recruitment.xlsx", "excel/lmia.xlsx", "excel/er.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())


class RecruitmnetSummaryDocxAdaptor:
    def __init__(self, recruitment_summary_obj: RecruitmnetSummaryModel):
        self.recruitment_summary_obj = recruitment_summary_obj

    def re_generate_dict(self):
        summary_info = {
            "resume_num": self.recruitment_summary_obj.summary.resume_num,
            "canadian_num": self.recruitment_summary_obj.summary.canadian_num,
            "unknown_num": self.recruitment_summary_obj.summary.unknown_num,
            "foreigner_num": self.recruitment_summary_obj.summary.foreigner_num,
            "total_canadian": self.recruitment_summary_obj.summary.total_canadian,
            "total_interviewed_canadians": self.recruitment_summary_obj.summary.total_interviewed_canadians,
            "canadian_records": self.recruitment_summary_obj.summary.canadian_records,
            "contact": self.recruitment_summary_obj.selected_contact,
            "advertisement": self.recruitment_summary_obj.advertisements,
        }
        return {**self.recruitment_summary_obj.dict(), **summary_info}

    def make(self, output_docx, template_no=None):
        template_path = os.path.abspath(
            os.path.join(DATADIR, "word/lmia_recruitment_summary.docx")
        )
        wm = WordMaker(template_path, self.re_generate_dict(), output_docx)
        wm.make()



from pydantic import BaseModel, EmailStr
from model.common.user import User
from typing import List


def user_schema(user):
    return {
        "_id": str(user["_id"]),
        "username": user["username"],
        "email": user["email"],
        "phone": user["phone"],
        # "password": user["password"],
        "role": user.get("role"),
        "permission": user.get("permission"),
    }


def users_schema(users):
    return [user_schema(user) for user in users]


class ShowUser(BaseModel):
    _id: str
    username: str
    email: EmailStr
    phone: str
    password: str
    role: str
    permission: list


class Login(BaseModel):
    email: EmailStr
    password: str


class Token(BaseModel):
    access_token: str
    token_type: str


class TokenData(BaseModel):
    # email: str | None = None
    email:str


class Permission(BaseModel):
    user_email: EmailStr
    permissions: List[str]
    remove: bool


class Role(BaseModel):
    user_email: EmailStr
    role: str



from pydantic import BaseModel
from typing import List
from utils.utils import formatMoney


class Finance(BaseModel):
    year: int
    total_asset: float
    net_asset: float
    revenue: float
    net_income: float
    retained_earning: float

    @property
    def formatted_revenue(self):
        return formatMoney(self.revenue)

    @property
    def formatted_net_income(self):
        return formatMoney(self.net_income)

    @property
    def formatted_retained_earning(self):
        return formatMoney(self.retained_earning)


class Finances:
    def __init__(self, finances: List[Finance]) -> None:
        self.finances = finances

    @property
    def last_income(self):
        if len(self.finances) > 0:
            return self.finances[0].revenue



from model.common.commonmodel import CommonModel, BuilderModel
from datetime import date
from pydantic import BaseModel, root_validator
from typing import List
from model.common.mixins import DatePeriod
from model.common.utils import checkRow


class Travel(DatePeriod):
    length: int
    destination: str
    purpose: str

    @root_validator
    def checkCompletion(cls, values):
        all_fields = ["start_date", "end_date", "length", "destination", "purpose"]
        required_fields = ["start_date", "end_date", "length", "destination", "purpose"]
        checkRow(values, all_fields, required_fields)
        return values


class Personal(BaseModel):
    last_name: str
    first_name: str


class M5562Model(BaseModel):
    travel: List[Travel]
    personal: Personal


class M5562ModelE(CommonModel, M5562Model):
    def __init__(self, excels=None, output_excel_file=None):
        mother_excels = ["excel/pa.xlsx"]
        super().__init__(excels, output_excel_file, mother_excels, globals())



from tkinter import Label
from webform.formcontrol import WebElement
from webform.webcommon import Page
from model.common.rcic import Rcics


class Representative:
    def __init__(self, rep: object, rcic_id_name):
        self.rep = rep
        self.we = WebElement()
        self.rcic = Rcics(self.rep.rciclist).getRcicByIdName(rcic_id_name)

    def fill(self):
        return [self.gotoRepPage, self.repPage]

    @property
    def gotoRepPage(self):
        page_actions = [
            self.we.buttonElement(
                "#navigation__list > li:nth-child(5) > a", label="Representative"
            ),
        ]
        next_page_tag = "#lastName"
        page = Page(
            page_actions,
            "body > div > main > div.layout-container > div > div > a",
            next_page_tag=next_page_tag,
            label="Add representative",
        )
        return page

    @property
    def repPage(self):
        page_actions = self.repInfo + self.repType + self.repAuth
        next_page_tag = (
            "body > div > main > div.layout-container > div > div > a:nth-child(3)"
        )
        return Page(
            page_actions,
            "#form > div > div > input",
            next_page_tag,
            label="Save representative",
        )

    @property
    def repInfo(self):
        return [
            self.we.inputElement("#lastName", self.rcic.last_name, label="Last name"),
            self.we.inputElement(
                "#firstName", self.rcic.first_name, label="First name"
            ),
            self.we.inputElement(
                "#organization", self.rcic.employer_legal_name, label="Employer name"
            ),
            self.we.inputElement("#phone", self.rcic.phone, label="Phone"),
            self.we.inputElement("#phoneSecondary", "", "Secondary phone"),
            self.we.inputElement("#email", self.rcic.email, label="Email"),
            self.we.selectElement("#country", self.rcic.country, label="Country"),
            self.we.inputElement("#addressLine", self.rcic.line1, label="Address line"),
            self.we.inputElement("#city", self.rcic.city, label="City"),
            self.we.inputElement("#state", self.rcic.province, label="Province"),
            self.we.inputElement("#postalCode", self.rcic.post_code, label="Post code"),
        ]

    @property
    def repType(self):
        return [
            self.we.radioElement(
                "#form > fieldset:nth-child(7) > div > div:nth-child(1) > div > div:nth-child(3) > label > input[type=radio]",
                label="Paid",
            ),
            self.we.radioElement(
                "#form > fieldset:nth-child(7) > div > div:nth-child(2) > div > div:nth-child(3) > label > input[type=radio]",
                label="Yes ICCRC",
            ),
            self.we.inputElement(
                "#crcMembershipId", self.rcic.rcic_number, label="RCIC number"
            ),
        ]

    @property
    def repAuth(self):
        return [
            self.we.checkboxElement(
                "#form > fieldset:nth-child(8) > div > div:nth-child(2) > label > input[type=checkbox]:nth-child(2)",
                True,
                label="Authorise Rep",
            ),
            self.we.checkboxElement(
                "#form > fieldset:nth-child(8) > div > div:nth-child(3) > label > input[type=checkbox]:nth-child(2)",
                True,
                label="Authorise BCPNP",
            ),
            # self.we.uploadElement("#form > fieldset:nth-child(8) > div > div.row > div > div.input-group.file-wrapper > div > button",rep_ee,label="Upload rep form for employee")
            # self.we.uploadElement("#form > fieldset:nth-child(9) > div > div > div > div.input-group.file-wrapper > div > button",rep_er,label="Upload rep form for employer")
        ]



from datetime import datetime, date
from enum import Enum, unique
from difflib import Differ
import re
from fuzzywuzzy import fuzz
from termcolor import colored
from tabulate import tabulate
from typing import Union, List
from rich.table import Table
from json import JSONEncoder
from datetime import date,datetime

# This function used to check a list's continuity of date. The list must have two columns as start date and end date.
# The fromat must be "yyyy-mm-dd" or "yyy-mm".  The list has to be from most recent to past. You can force sort it by assign sort =True
# The output will be True or False, plus sorted input list and message.
def checkContinuity(data_set: list, sort=False):
    if sort:
        data_set = sorted(data_set, key=lambda x: (x[0], x[1]), reverse=True)
    ok = []
    errors = []

    def difference(l1, l2):
        if type(l1) == date:
            l1 = l1.strftime("%Y-%m-%d")
        if type(l2) == date:
            l2 = l2.strftime("%Y-%m-%d")

        if l2 == None or l2 == "Present":
            l2 = (
                datetime.today().strftime("%Y-%m-%d")
                if bool(re.search(r"^\d{4}-\d{2}-\d{2}", l1))
                else datetime.today().strftime("%Y-%m")
            )
        if bool(re.search(r"^\d{4}-\d{2}-\d{2}", l1)) and bool(
            re.search(r"^\d{4}-\d{2}-\d{2}", l2)
        ):
            current_start = datetime.strptime(l1, "%Y-%m-%d")
            previous_end = datetime.strptime(l2, "%Y-%m-%d")
            return (current_start - previous_end).days, "day(s)"
        if bool(re.search(r"^\d{4}-\d{2}", l1)) and bool(
            re.search(r"^\d{4}-\d{2}", l2)
        ):
            y1, m1 = l1.split("-")
            y2, m2 = l2.split("-")
            return (int(y1) - int(y2)) * 12 + int(m1) - int(m2), "month(s)"

    for i in range(len(data_set) - 1):
        diff, unit = difference(data_set[i][0], data_set[i + 1][1])
        if diff > 1:
            ok.append(False)
            errors.append(
                f"{'List sorted. ' if sort else ''}The date is not continious between line {i+1} to {i+2} (missed {diff-1} {unit})"
            )
        elif diff < 0:
            ok.append(False)
            errors.append(
                f"{'List sorted. ' if sort else ''}The date is not continious between line {i+1} to {i+2} (overlapped {diff} {unit})"
            )
        else:
            ok.append(True)
    if all(ok):
        return True, data_set, "date is continious"
    else:
        return False, data_set, errors


#  Used to find different between two text strings. Initialize with tc=TextChanged(t1,t2), and get differences by tc.text_added or tc.text_deleted
class TextChanged:
    def __init__(self, text1, text2):
        self.text1 = text1
        self.text2 = text2

    def __get_changed(self):

        t1_lines = self.text1.splitlines()
        t2_lines = self.text2.splitlines()

        diff = Differ().compare(t1_lines, t2_lines)
        changed = list(diff)

        added = r"^\+\s(.+)"
        deleted = r"^\-\s(.+)"
        text_added = [
            d[2:] for d in changed if re.findall(added, d)
        ]  # d[2:] delete '+ ' at the begining
        text_added = "\n".join(text_added)
        text_deleted = [d[2:] for d in changed if re.findall(deleted, d)]
        text_deleted = "\n".join(text_deleted)

        return {"text_added": text_added, "text_deleted": text_deleted}

    @property
    def text_added(self):
        return self.__get_changed()["text_added"]

    @property
    def text_deleted(self):
        return self.__get_changed()["text_deleted"]


def best_match(term, list_names, min_score=0, return_score=False):
    max_score = -1
    max_name = ""
    for term2 in list_names:
        score = fuzz.ratio(term, term2)
        if (score > min_score) and (score > max_score):
            max_name = term2
            max_score = score
    if return_score:
        return max_name, max_score
    else:
        return max_name


def append_ext(filename: str | List[str], ext):
    def convert(fn):
        names = fn.split(".")
        if len(names) == 2:
            return fn
        elif len(names) == 1:
            return names[0] + ext
        else:
            raise ValueError(f"{filename} is invalid filename")

    if type(filename) == list:
        return [convert(fn) for fn in filename]
    else:
        return convert(filename)


def remove_ext(filename: Union[str, List[str]]):
    def convert(fn):
        names = fn.split(".")
        if len(names) >= 1:
            return names[0]
        else:
            raise ValueError(f"{filename} is invalid filename")

    if type(filename) == list:
        return [convert(fn) for fn in filename]
    else:
        return convert(filename)


Month = Enum(
    "Month",
    (
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec",
    ),
)


@unique
class Sex(Enum):
    Female = 0
    Male = 1


# input list and outpout markdown table text
def markdown_table(ds: list):
    head = ds[0]
    head_str = "|"
    head_sep = "|"
    for col in range(0, len(ds[0])):
        head_str += str(head[col]) + "|"
        head_sep += "------------- " + "|"
    # print(head_str)
    # print(head_sep)
    output = head_str + "\n" + head_sep + "\n"

    row_data = "|"
    for row in range(1, len(ds)):
        for col in range(0, len(ds[0])):
            row_data += str(ds[row][col]) + "|"
        # print(row_data)
        output += row_data + "\n"
        row_data = "|"

    return output


# Some common input functions
def multi_line_input():
    print("Ctrl-D  to save it.")
    contents = []
    while True:
        try:
            line = input()
        except EOFError:
            break
        contents.append(line)
    return "\n".join(contents)


# 打印1D的列表为多行
def printList(ls, index=True, sep="\t"):  # 打印list，默认在前面加上index，用tab分割
    i = 0
    for l in ls:
        item = f"{i}{sep}{l}" if index == True else f"{l}"
        print(item)
        i += 1


# 将列表变成一行字符串
def ListInline(ls, sep="\t"):
    item = ""
    for l in ls:
        item = item + f"{l}{sep}"
    return item


# 打印2D的列表
def printList2D(ls, index=False, sep="\t"):
    i = 0
    item = ""
    for l in ls:
        line = ListInline(l, sep)
        item = f"{i}{sep}{line}" if index == True else f"{line}{sep}"
        print(item)
        i += 1


# 使用tabulate 打印formatted 2D的list为表格
def printFList2D(ls, index=False):
    i = 0
    items = []
    for l in ls:
        if index == True:
            l.insert(0, i)
            items.append(l)

        else:
            items.append(l)
        i += 1
    print(tabulate(items))


# formatter of money
def formatMoney(number: float, currency_type="$"):
    return currency_type + "{:,.2f}".format(number)


# join string as speaking language
def speaking_list(l: list):
    if len(l) == 1:
        result = l[0]
    elif len(l) == 2:
        result = " and ".join(l)
    elif len(l) > 2:
        result = ", ".join(l[:-1]) + " and " + l[-1]
    else:
        result = ""
    return result


# make table based on rich
def makeTable(table_name, table_list: list):
    table = Table(title=table_name)
    titles = table_list.pop(0)
    for title in titles:
        table.add_column(title, justify="left", style="cyan", no_wrap=False)
    for content in table_list:
        table.add_row(*content)

    return table


class DateEncoder(JSONEncoder):
        #Override the default method
        def default(self, obj):
            if isinstance(obj, (date)):
                return obj.strftime("%Y-%m-%d")

def main():
    # print(append_ext("a.docx", ".xlsx"))
    print(formatMoney(9211223213.22))


if __name__ == "__main__":
    main()



from turtle import position
from model.common.advertisement import Advertisements, InterviewRecords
from pdfform.jsonmaker import JsonMaker
from model.common.address import Addresses

# from model.bcpnp.jobofferform import JobOfferFormModel
from model.common.contact import Contacts
from assess.policy.bcpnp.data import bc_tech_pilot
from datetime import date
import json
from termcolor import colored


class FormBuilderEmployerDeclaration:
    """Form builder"""

    def __init__(self, jof: object):
        self.jof = jof
        self.form = JsonMaker()
        self.text_speed = 0.01

    def start(self):
        self.form.add_skip(1)

    def employee(self):
        self.form.add_text(self.jof.personal.last_name, pause=self.text_speed)
        self.form.add_text(self.jof.personal.first_name, pause=self.text_speed)

    def employer(self):
        self.form.add_text(self.jof.general.legal_name, pause=self.text_speed)
        self.form.add_text(self.jof.general.operating_name or "", pause=self.text_speed)

        mailing_address = Addresses(self.jof.eraddress).mailing
        business_address = Addresses(self.jof.eraddress).business

        self.form.add_text(mailing_address.line1, pause=self.text_speed)
        self.form.add_text(mailing_address.city, pause=self.text_speed)
        self.form.add_text(mailing_address.province, pause=self.text_speed)
        self.form.add_text(mailing_address.country, pause=self.text_speed)
        self.form.add_text(mailing_address.post_code, pause=self.text_speed)

        if mailing_address == business_address:
            self.form.add_skip(5)
        else:
            self.form.add_text(business_address.line1, pause=self.text_speed)
            self.form.add_text(business_address.city, pause=self.text_speed)
            self.form.add_text(business_address.province, pause=self.text_speed)
            self.form.add_text(business_address.country, pause=self.text_speed)
            self.form.add_text(business_address.post_code, pause=self.text_speed)
        # contact
        contact = Contacts(self.jof.contact).primary
        self.form.add_text(contact.last_name, pause=self.text_speed)
        self.form.add_text(contact.first_name, pause=self.text_speed)
        self.form.add_text(contact.position, pause=self.text_speed)
        self.form.add_text(contact.phone, pause=self.text_speed)
        self.form.add_text(contact.email, pause=self.text_speed)

        # other
        self.form.add_text(self.jof.general.website, pause=self.text_speed)
        self.form.add_text(
            str(self.jof.general.ft_employee_number), pause=self.text_speed
        )
        self.form.add_text(
            self.jof.general.establish_date.strftime("%d/%m/%y"), pause=self.text_speed
        )

        self.form.add_text(self.jof.general.industry, pause=self.text_speed)

        if self.jof.general.corporate_structure == "Incorporated":
            self.form.add_checkbox(True)
            self.form.add_skip(2)
        elif self.jof.general.corporate_structure == "Limited Liability Partnership":
            self.form.add_skip(1)
            self.form.add_checkbox(True)
            self.form.add_skip(1)
        elif self.jof.general.corporate_structure == "Extra-provincially-registered":
            self.form.add_skip(2)
            self.form.add_checkbox(True)
        self.form.add_text(self.jof.general.registration_number, pause=self.text_speed)
        # other corporate structure
        if (
            self.jof.general.corporate_structure == "federally-incorporated"
            or self.jof.general.corporate_structure == "Other"
        ):

            self.form.add_skip(4)
            self.form.add_checkbox(True)
            self.form.add_text(
                self.jof.general.corporate_structure, pause=self.text_speed
            )
        else:
            self.form.add_skip(2)

    def joboffer(self):
        self.form.add_text(self.jof.joboffer.job_title, pause=self.text_speed)
        self.form.add_text(self.jof.joboffer.hourly_rate, pause=self.text_speed)
        self.form.add_text(self.jof.joboffer.annual_rate, pause=self.text_speed)
        self.form.add_text(self.jof.joboffer.weekly_hours, pause=self.text_speed)

        work_locations = Addresses(self.jof.eraddress).workings
        # first work location
        if len(work_locations) < 1:
            raise ValueError("Work location missed")
        self.form.add_text(work_locations[0].line1, pause=self.text_speed)
        self.form.add_text(work_locations[0].city, pause=self.text_speed)
        self.form.add_text(work_locations[0].post_code, pause=self.text_speed)
        self.form.add_text(work_locations[0].phone, pause=self.text_speed)
        if len(work_locations) >= 2:
            self.form.add_text(work_locations[1].line1, pause=self.text_speed)
            self.form.add_text(work_locations[1].city, pause=self.text_speed)
            self.form.add_text(work_locations[1].post_code, pause=self.text_speed)
            self.form.add_text(work_locations[1].phone, pause=self.text_speed)
        else:
            self.form.add_skip(4)

    def tech(self):
        if self.jof.joboffer.noc in bc_tech_pilot:
            if self.jof.joboffer.permanent:
                self.form.add_radio(True)
                self.form.add_skip(5)  # messy skip :(
            else:
                self.form.add_radio(False)
                self.form.add_text(
                    self.jof.joboffer.work_end_date.strftime("%d-%b-%Y"),
                    pause=self.text_speed,
                )
                self.form.add_skip(3)
                self.form.add_text(
                    self.jof.joboffer.why_not_permanent, pause=self.text_speed
                )
        else:
            self.form.add_skip(5)

    def position(self):
        self.form.add_info("3c. Position details")
        # new position
        self.form.add_radio_list(
            pause=0.1,
            position=1
            if self.jof.position.is_new
            else 2,  # here 1 and 2 is special, normally should be 0 or 1
        )
        self.form.add_radio_list(
            pause=0.1, position=1 if self.jof.position.under_cba else 2
        )
        if self.jof.position.under_cba:
            self.form.add_text(self.jof.position.which_union, pause=self.text_speed)
        # else:
        #     self.form.add_skip(1)

        # employee number in same position
        self.form.add_text(
            self.jof.position.has_same_number
            if self.jof.position.has_same_number
            else 0,
            pause=self.text_speed,
        )
        self.form.add_text(
            self.jof.position.vacancies_number
            if self.jof.position.vacancies_number
            else 0,
            pause=self.text_speed,
        )
        self.form.add_text(
            self.jof.position.laidoff_with12 if self.jof.position.laidoff_with12 else 0,
            pause=self.text_speed,
        )
        self.form.add_text(
            self.jof.position.laidoff_current
            if self.jof.position.laidoff_current
            else 0,
            pause=self.text_speed,
        )
        # language requirement
        self.form.add_radio_list(
            position=0 if self.jof.joboffer.other_language_required else 1
        )
        if self.jof.joboffer.other_language_required:
            self.form.add_text(
                self.jof.joboffer.reason_for_other, pause=self.text_speed
            )
        else:
            self.form.add_skip(1)

        # lmia status
        self.form.add_radio_list(position=0 if self.jof.position.lmia_refused else 1)
        if self.jof.position.lmia_refused:
            self.form.add_text(
                self.jof.position.lmia_refused_reason, pause=self.text_speed
            )
        else:
            self.form.add_skip(1)

        # license issue
        if self.jof.joboffer.license_request:
            if self.jof.joboffer.license_met:
                self.form.add_radio_list(position=0)
                self.form.add_text(
                    self.jof.joboffer.license_met_reason, pause=self.text_speed
                )
            else:
                self.form.add_radio_list(position=1)
                self.form.add_text(
                    self.jof.joboffer.license_met_reason, pause=self.text_speed
                )
        else:
            self.form.add_radio_list(position=2)

    def recruit(self):
        ads = Advertisements(self.jof.advertisement)
        interviews = InterviewRecords(self.jof.interviewrecord)
        recruited = True if ads.amount > 0 else False
        if recruited:
            self.form.add_radio(True)
            self.form.add_text(interviews.resume_num, pause=self.text_speed)
            self.form.add_text(ads.min_days, pause=self.text_speed)
            self.form.add_text(interviews.summary, pause=self.text_speed)
            self.form.add_text(
                self.jof.personalassess.why_qualified_say, pause=self.text_speed
            )
            self.form.add_text(
                self.jof.recruitmentsummary.reasons_not_hire_canadians,
                pause=self.text_speed,
            )
            self.form.add_skip(1)
        else:
            self.form.add_radio(False)
            self.form.add_skip(6)

    def sign(self):
        contact = Contacts(self.jof.contact).primary
        # messy skip in pdf
        self.form.add_text(contact.last_name, pause=self.text_speed)
        self.form.add_text(contact.position, pause=self.text_speed)
        self.form.add_skip(21)
        self.form.add_text(contact.first_name, pause=self.text_speed)
        self.form.add_text(date.today().strftime("%d-%b-%Y"), pause=self.text_speed)

    def get_form(self):
        self.start()
        self.employee()
        self.employer()
        self.joboffer()
        self.tech()
        self.position()
        self.recruit()
        self.sign()
        return self.form

    def save(self, flename: str):
        actions = self.get_form().actions
        with open(flename, "w") as output:
            json.dump(actions, output, indent=3, default=str)
        print(colored(f"{flename} has been created", "green"))



from passlib.hash import pbkdf2_sha256


class Hash:
    @classmethod
    def hash(cls, password: str):
        return pbkdf2_sha256.hash(password)

    @classmethod
    def verify(cls, plain_password: str, hashed_password: str):
        return pbkdf2_sha256.verify(plain_password, hashed_password)



from webform.webcommon import WebPages, Page


class Layoff(WebPages):
    @property
    def actions(self):
        return [self.page1, self.page2, self.page3, self.page4]

    @property
    def page1(self):
        lead2layoff = (
            [
                self.web_element.inputElement(
                    "#\\39 515 > input",
                    str(self.app.lmi.laid_off_canadians),
                    label="number of laid off Canadians",
                ),
                self.web_element.inputElement(
                    "#\\39 516 > input",
                    str(self.app.lmi.laid_off_tfw),
                    label="number of laid off TFWs",
                ),
                self.web_element.areatextElement(
                    "#\\39 517 > textarea",
                    self.app.lmi.laid_off_reason,
                    label="reason for lay-off",
                ),
            ]
            if self.app.lmi.laid_off_in_12
            else []
        )

        lead2jobloss_id = (
            "#\\39 518 > div > input:nth-child(1)"
            if self.app.lmi.canadian_lost_job
            else "#\\39 518 > div > input:nth-child(4)"
        )
        lead2joblosss = self.web_element.radioElement(
            lead2jobloss_id, label="Leads to lay off?"
        )

        page_actions = [*lead2layoff, lead2joblosss]
        next_page_tag = (
            "#\\39 520 > div > input:nth-child(4)"  # receiving work-sharing ? No-id
        )

        return Page(page_actions, "#next", next_page_tag, label="lay off").page

    @property
    def page2(self):
        layoff_details = (
            [
                self.web_element.areatextElement(
                    "#\\39 519 > textarea",
                    self.app.lmi.canadian_lost_job_info,
                    label="details of leading to layoff",
                )
            ]
            if self.app.lmi.canadian_lost_job
            else []
        )

        workshaing_id = (
            "#\\39 520 > div > input:nth-child(1)"
            if self.app.lmi.is_work_sharing
            else "#\\39 520 > div > input:nth-child(4)"
        )
        worksharing = self.web_element.radioElement(
            workshaing_id, label="Has work sharing?"
        )

        page_actions = [*layoff_details, worksharing]
        next_page_tag = "#\\39 522 > div > input:nth-child(4)"  # is there any labour dispute ? No-id

        return Page(
            page_actions, "#next", next_page_tag, label="leading to job loss"
        ).page

    @property
    def page3(self):
        page_actions = [
            self.web_element.radioElement(
                "#\\39 522 > div > input:nth-child(1)"
                if self.app.lmi.labour_dispute
                else "#\\39 522 > div > input:nth-child(4)",
                label="has labour dispute?",
            )
        ]

        next_page_tag = "#documents"  # Documents button to uploading files

        return Page(page_actions, "#next", next_page_tag, label="labour dispute").page

    @property
    def page4(self):
        page_actions = (
            [
                self.web_element.areatextElement(
                    "#\\39 523 > textarea",
                    self.app.lmi.labour_dispute_info,
                    label="Labour dispute details",
                )
            ]
            if self.app.lmi.labour_dispute
            else []
        )

        next_page_tag = "#uploadDocument"  # Documents upload button to uploading files

        return Page(
            page_actions, "#documents", next_page_tag, label="labour dispute details"
        ).page
